maker_config_string = """
#-----Genome (these are always required)
genome=%s #genome sequence (fasta file or fasta embeded in GFF3 file)
organism_type=eukaryotic #eukaryotic or prokaryotic. Default is eukaryotic


#-----Re-annotation Using MAKER Derived GFF3
maker_gff= #MAKER derived GFF3 file
est_pass=0 #use ESTs in maker_gff: 1 = yes, 0 = no
altest_pass=0 #use alternate organism ESTs in maker_gff: 1 = yes, 0 = no
protein_pass=0 #use protein alignments in maker_gff: 1 = yes, 0 = no
rm_pass=0 #use repeats in maker_gff: 1 = yes, 0 = no
model_pass=0 #use gene models in maker_gff: 1 = yes, 0 = no
pred_pass=0 #use ab-initio predictions in maker_gff: 1 = yes, 0 = no
other_pass=0 #passthrough anyything else in maker_gff: 1 = yes, 0 = no


#-----EST Evidence (for best results provide a file for at least one)
est=%s #set of ESTs or assembled mRNA-seq in fasta format
altest=%s #EST/cDNA sequence file in fasta format from an alternate organism
est_gff=%s #aligned ESTs or mRNA-seq from an external GFF3 file
altest_gff=%s #aligned ESTs from a closly relate species in GFF3 format


#-----Protein Homology Evidence (for best results provide a file for at least one)
protein=%s #protein sequence file in fasta format (i.e. from mutiple oransisms)
protein_gff=%s #aligned protein homology evidence from an external GFF3 file


#-----Repeat Masking (leave values blank to skip repeat masking)
model_org=all #select a model organism for RepBase masking in RepeatMasker
rmlib=%s #provide an organism specific repeat library in fasta format for RepeatMasker
repeat_protein=%s #provide a fasta file of transposable element proteins for RepeatRunner
rm_gff=%s #pre-identified repeat elements from an external GFF3 file
prok_rm=0 #forces MAKER to repeatmask prokaryotes (no reason to change this), 1 = yes, 0 = no
softmask=1 #use soft-masking rather than hard-masking in BLAST (i.e. seg and dust filtering)


#-----Gene Prediction
snaphmm=%s #SNAP HMM file
gmhmm= #GeneMark HMM file
augustus_species=%s #Augustus gene prediction species model
fgenesh_par_file= #FGENESH parameter file
pred_gff= #ab-initio predictions from an external GFF3 file
model_gff= #annotated gene models from an external GFF3 file (annotation pass-through)
est2genome=%d #infer gene predictions directly from ESTs, 1 = yes, 0 = no
protein2genome=%d #infer predictions from protein homology, 1 = yes, 0 = no
trna=1 #find tRNAs with tRNAscan, 1 = yes, 0 = no
snoscan_rrna=%s #rRNA file to have Snoscan find snoRNAs
unmask=0 #also run ab-initio prediction programs on unmasked sequence, 1 = yes, 0 = no


#-----Other Annotation Feature Types (features MAKER doesn't recognize)
other_gff= #extra features to pass-through to final MAKER generated GFF3 file


#-----External Application Behavior Options
alt_peptide=C #amino acid used to replace non-standard amino acids in BLAST databases
cpus=1 #max number of cpus to use in BLAST and RepeatMasker (not for MPI, leave 1 when using MPI)


#-----MAKER Behavior Options
max_dna_len=100000 #length for dividing up contigs into chunks (increases/decreases memory usage)
min_contig=1 #skip genome contigs below this length (under 10kb are often useless)


pred_flank=200 #flank for extending evidence clusters sent to gene predictors
pred_stats=0 #report AED and QI statistics for all predictions as well as models
AED_threshold=1 #Maximum Annotation Edit Distance allowed (bound by 0 and 1)
min_protein=0 #require at least this many amino acids in predicted proteins
alt_splice=0 #Take extra steps to try and find alternative splicing, 1 = yes, 0 = no
always_complete=0 #extra steps to force start and stop codons, 1 = yes, 0 = no
map_forward=0 #map names and attributes forward from old GFF3 genes, 1 = yes, 0 = no
keep_preds=0 #Concordance threshold to add unsupported gene prediction (bound by 0 and 1)


split_hit=10000 #length for the splitting of hits (expected max intron size for evidence alignments)
single_exon=0 #consider single exon EST evidence when generating annotations, 1 = yes, 0 = no
single_length=250 #min length required for single exon ESTs if 'single_exon is enabled'
correct_est_fusion=0 #limits use of ESTs in annotation to avoid fusion genes


tries=2 #number of times to try a contig if there is a failure for some reason
clean_try=0 #remove all data from previous run before retrying, 1 = yes, 0 = no
clean_up=0 #removes theVoid directory with individual analysis files, 1 = yes, 0 = no
TMP=%s #specify a directory other than the system default temporary directory for temporary files

"""


def write_maker_config(config_file, **kwargs):
    defaults_parameter = {
        "genome": "",
        "est": "",
        "altest": "",
        "est_gff": "",
        "altest_gff": "",
        "protein": "",
        "protein_gff": "",
        "rmlib": "",
        "repeat_protein": "",
        "rm_gff": "",
        "snaphmm": "",
        "augustus_species": "",
        "est2genome": 0,
        "protein2genome": 0,
        "snoscan_rrna": "",
        "TMP": "",
    }

    for k in kwargs:
        if k in defaults_parameter:
            defaults_parameter[k] = kwargs[k]

    parameter_tuple = (
        defaults_parameter["genome"],
        defaults_parameter["est"],
        defaults_parameter["altest"],
        defaults_parameter["est_gff"],
        defaults_parameter["altest_gff"],
        defaults_parameter["protein"],
        defaults_parameter["protein_gff"],
        defaults_parameter["rmlib"],
        defaults_parameter["repeat_protein"],
        defaults_parameter["rm_gff"],
        defaults_parameter["snaphmm"],
        defaults_parameter["augustus_species"],
        defaults_parameter["est2genome"],
        defaults_parameter["protein2genome"],
        defaults_parameter["snoscan_rrna"],
        defaults_parameter["TMP"],
    )

    with open(config_file, 'w') as f:
        f.write(maker_config_string % parameter_tuple)


maker_evaluate_anno_shell = """#! /bin/bash

source activate maker_p

index_log=%s

fasta_merge -d $index_log
gff3_merge -d $index_log
gff3_merge -n -s -d $index_log > genome.noseq.gff
grep -P "\\tgene\\t" genome.all.gff |grep "noncoding" -v|awk '{ sum += ($5 - $4) } END { print NR, sum / NR }' > gene.length
perl ~/Program/bin_xyx/AED_cdf_generator.pl -b 0.025 genome.all.gff  > AED.stat

conda deactivate

source activate busco4

busco -c 56 -m proteins -i genome.all.maker.proteins.fasta -o proteins_vs_embryophyta_odb10 -l embryophyta_odb10
rm -rf busco_downloads

conda deactivate
"""

def write_maker_evaluate_anno_shell(shell_file, index_log_file):
    with open(shell_file, 'w') as f:
        f.write(maker_evaluate_anno_shell % index_log_file)


maker_to_snap_augustus_shell = """#! /bin/bash

tag=$1
master_datastore_index_log=$2

source activate maker_p

# pae_r1
# Pae.frag.1_master_datastore_index.log


#############snap#################


# export 'confident' gene models from MAKER and rename to something meaningful
maker2zff -x 0.25 -l 50 -d $master_datastore_index_log 
# 20 min
file_prefix=${tag}.zff.length50_aed0.25
rename genome $file_prefix *


# gather some stats and validate
fathom $file_prefix.ann $file_prefix.dna -gene-stats > gene-stats.log 2>&1    
# 3 min
fathom $file_prefix.ann $file_prefix.dna -validate > validate.log 2>&1    
# 3 min


# collect the training sequences and annotations, plus 1000 surrounding bp for training
fathom $file_prefix.ann $file_prefix.dna -categorize 1000 > categorize.log 2>&1    
# 3 min
fathom uni.ann uni.dna -export 1000 -plus > uni-plus.log 2>&1    
# 1 min


# create the training parameters
mkdir params
cd params
forge ../export.ann ../export.dna > ../forge.log 2>&1    
# 1 min
cd ..


# assembly the HMM
hmm-assembler.pl $file_prefix params > $file_prefix.hmm    
# 1min


#############augustus#################

perl ~/Program/bin_xyx/zff2augustus_gbk.pl export.ann export.dna > augustus.gbk
new_species.pl --species=$tag
randomSplit.pl augustus.gbk 100
etraining --species=$tag augustus.gbk.train


# test
# augustus --species=$tag augustus.gbk.test | tee firsttest.out


# 迭代
export OMP_NUM_THREADS=1
optimize_augustus.pl --cpus=56 --kfold=56 --species=$tag augustus.gbk.train    
# 29 h

"""

def write_maker_to_snap_augustus_shell(shell_file):
    with open(shell_file, 'w') as f:
        f.write(maker_to_snap_augustus_shell)



inherit_maker_gff_shell = """#! /bin/bash
# transcript alignments
awk '{ if ($2 == "est2genome") print $0 }' genome.noseq.gff > for_next.est2genome.gff
# protein alignments
awk '{ if ($2 == "protein2genome") print $0 }' genome.noseq.gff > for_next.protein2genome.gff
# repeat alignments
awk '{ if ($2 ~ "repeat") print $0 }' genome.noseq.gff > for_next.repeats.gff
"""

def write_inherit_maker_gff_shell(shell_file):
    with open(shell_file, 'w') as f:
        f.write(inherit_maker_gff_shell)



import configparser
import logging
import time
import os
import pickle
from operator import attrgetter

__author__ = 'Yuxing Xu'


def printer_list(list_input, sep="\t", head="", wrap_num=None):
    """
    make a list to a string with a given sep chara and head
    :param list_input: list you give me
    :param sep: Split character like "," or "\t"
    :param head: head for the output string
    :return: string
    """
    printer = head
    num = 0

    for i in list_input:
        printer = printer + str(i) + sep
        num = num + 1
        if wrap_num:
            if num % wrap_num == 0:
                printer = printer + "\n"

    printer = printer.rstrip(sep)
    return printer


def mulit_sort(xs, specs):
    """
    class Student:
        def __init__(self, name, grade, age):
            self.name = name
            self.grade = grade
            self.age = age
        def __repr__(self):
            return repr((self.name, self.grade, self.age))

    student_objects = [
        Student('john', 'A', 15),
        Student('jane', 'B', 12),
        Student('dave', 'B', 10),
    ]

    multisort(list(student_objects), (('grade', True), ('age', False)))
    """
    for key, reverse in reversed(specs):
        xs.sort(key=attrgetter(key), reverse=reverse)
    return xs


def logging_init(program_name, log_file=None):
    # create logger with 'program_name'
    logger = logging.getLogger(program_name)
    logger.setLevel(logging.DEBUG)
    # create formatter and add it to the handlers
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    if not log_file is None:
        # create file handler which logs even debug messages
        fh = logging.FileHandler(log_file)
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        logger.addHandler(fh)
    # create console handler with a higher log level
    ch = logging.StreamHandler()
    ch.setLevel(logging.ERROR)
    ch.setFormatter(formatter)
    # add the handlers to the logger
    logger.addHandler(ch)

    return logger


def type_check(p_name, p_value, parameter_type_dict):
    if parameter_type_dict is None or p_name not in parameter_type_dict:
        return p_name, p_value
    else:
        if parameter_type_dict[p_name] == 'str':
            return p_name, str(p_value)
        elif parameter_type_dict[p_name] == 'int':
            return p_name, int(p_value)
        elif parameter_type_dict[p_name] == 'float':
            return p_name, float(p_value)
        elif parameter_type_dict[p_name] == 'bool':
            return p_name, bool(p_value)
        else:
            raise ValueError('Unknown type %s' % parameter_type_dict[p_name])


def configure_parser(arg_obj, defaults_config_file=None, config_file=None, parameter_type_dict=None,
                     parameter_parser_block=None):
    """
    config_file should be ini file

    [Simple Values]
    key=value
    spaces in keys=allowed
    spaces in values=allowed as well
    spaces around the delimiter = obviously
    you can also use : to delimit keys from values

    [All Values Are Strings]
    values like this: 1000000
    or this: 3.14159265359
    are they treated as numbers? : no
    integers, floats and booleans are held as: strings
    can use the API to get converted values directly: true

    [Multiline Values]
    chorus: I'm a lumberjack, and I'm okay
        I sleep all night and I work all day

    [No Values]
    key_without_value
    empty string value here =

    [You can use comments]
    # like this
    ; or this

    # By default only in an empty line.
    # Inline comments can be harmful because they prevent users
    # from using the delimiting characters as parts of values.
    # That being said, this can be customized.

        [Sections Can Be Indented]
            can_values_be_as_well = True
            does_that_mean_anything_special = False
            purpose = formatting for readability
            multiline_values = are
                handled just fine as
                long as they are indented
                deeper than the first line
                of a value
            # Did I mention we can indent comments, too?

        :param arg_obj:

    args

    :param parameter_type_dict:

    parameter_type_dict = {"reference_genome": "str",
                       "work_dir": "str",
                       "db_file_fnf": "str",
                       "speci_tree_file": "str",
                       "taxonomy_dir": "str",
                       "target_speci": "str",
                       "prominence": "int",
                       "gap_limit": "int",
                       "min_ranges": "int",
                       "query_step": "int",
                       "query_length": "int",
                       "subseq_kmer_frequencies_thre": "int",
                       "threshold_node_level": "str",
                       "permutation_round": "int",
                       "seed": "int",
                       "p_value_thre": "float",
                       "evalue": "float",
                       "num_threads": "int"
                       }

    :param parameter_parser_block:

    """

    output_cfg_dict = {}

    # read defaults config
    if defaults_config_file is not None:
        def_cfg = configparser.ConfigParser()
        def_cfg.read(defaults_config_file)

        for cfg_block in def_cfg.sections():
            for key in def_cfg[cfg_block]:
                value = def_cfg[cfg_block][key]
                p_name, p_value = type_check(key, value, parameter_type_dict)
                output_cfg_dict[p_name] = p_value

    # read given config
    if config_file is not None:
        cfg = configparser.ConfigParser()
        cfg.read(config_file)

        for cfg_block in cfg.sections():
            for key in cfg[cfg_block]:
                if (parameter_parser_block is not None) and (cfg_block not in parameter_parser_block):
                    continue
                value = cfg[cfg_block][key]
                p_name, p_value = type_check(key, value, parameter_type_dict)
                # if p_name not in output_cfg_dict:
                #     raise ValueError("unknown parameter %s" % p_name)
                # else:
                #     output_cfg_dict[p_name] = p_value
                output_cfg_dict[p_name] = p_value

    # read command arg
    for p_name in output_cfg_dict:
        if hasattr(arg_obj, p_name) and (getattr(arg_obj, p_name) is not None):
            output_cfg_dict[p_name] = getattr(arg_obj, p_name)

    # output to args
    for p_name in output_cfg_dict:
        setattr(arg_obj, p_name, output_cfg_dict[p_name])

    return arg_obj


def pickle_dump(save_object, output_pickle_file):
    try:
        OUT = open(output_pickle_file, 'wb')
        pickle.dump(save_object, OUT)
        OUT.close()
        return output_pickle_file
    except:
        raise ValueError("Failed to write %s" % output_pickle_file)


def pickle_load(input_pickle_file):
    try:
        TEMP = open(input_pickle_file, 'rb')
        output_object = pickle.load(TEMP)
        TEMP.close()
        return output_object
    except:
        raise ValueError("Failed to open %s" % input_pickle_file)


def pickle_step(function, input_args_list, output_pickle_file):
    """
    Sometimes the result of a function may have already been run and saved, so I can try to read the result and run the function again if it doesn't work
    """

    if os.path.exists(output_pickle_file):
        try:
            TEMP = open(output_pickle_file, 'rb')
            output_object = pickle.load(TEMP)
            TEMP.close()
            return output_object
        except:
            pass
    output_object = function(*input_args_list)
    OUT = open(output_pickle_file, 'wb')
    pickle.dump(output_object, OUT)
    OUT.close()

    return output_object


def time_now():
    time_tmp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
    return time_tmp


def dict_key_value_transpose(dict_list):
    dict_hash = {}
    for i in dict_list:
        for j in dict_list[i]:
            if j not in dict_hash:
                dict_hash[j] = []
            dict_hash[j].append(i)
    return dict_hash



import re
import subprocess
import time
import os
import shutil
from multiprocessing import Pool, TimeoutError
from multiprocessing.dummy import Pool as ThreadPool
from functools import partial
from lib.common.util import printer_list, logging_init, time_now

__author__ = 'Yuxing Xu'


def mkdir(dir_name, keep=False):
    if keep is False:
        if os.path.exists(dir_name):
            shutil.rmtree(dir_name)
        os.makedirs(dir_name)
    else:
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)

    return dir_name


def rmdir(dir_name):
    if os.path.exists(dir_name):
        if os.path.isdir(dir_name):
            shutil.rmtree(dir_name)
        else:
            os.remove(dir_name)


def have_file(file_name, null_is_ok=False):
    if os.path.exists(file_name):
        if null_is_ok:
            return True
        elif os.path.getsize(file_name):
            return True
    else:
        return False


def get_file_name(give_path):
    return give_path.split("/")[-1]


def get_file_dir(file_path):
    abs_file_path = os.path.abspath(file_path)
    return os.path.dirname(abs_file_path)


def copy_file(source_path, target_path, keep=False):
    source_path = os.path.abspath(source_path)
    target_path = os.path.abspath(target_path)

    new_path = shutil.copy(source_path, target_path)

    return new_path


def ln_file(source_path, target_path):

    if os.path.isdir(target_path):
        target_path = target_path + "/" + get_file_name(source_path)

    os.symlink(os.path.abspath(source_path), target_path)

    return target_path


def move_file(source_path, target_path, keep=False):
    source_path = os.path.abspath(source_path)
    target_path = os.path.abspath(target_path)

    new_path = shutil.move(source_path, target_path)

    return new_path


def gunzip_file(raw_file):
    raw_file = os.path.abspath(raw_file)

    cmd_string = "gunzip " + raw_file
    cmd_run(cmd_string, silence=True)

    gunzip_file = re.sub(".gz$", "", raw_file)

    return gunzip_file


def gzip_file(raw_file):
    raw_file = os.path.abspath(raw_file)

    cmd_string = "gzip " + raw_file
    cmd_run(cmd_string, silence=True)

    gzip_file = raw_file + ".gz"

    return gzip_file


def md5sum_check(file_name, original_md5):
    import hashlib

    if not os.path.exists(file_name):
        return False

    with open(file_name, "rb") as file_to_check:
        data = file_to_check.read()
        md5_returned = hashlib.md5(data).hexdigest()

    if original_md5 == md5_returned:
        return True
    else:
        return False


def remove_file_name_suffix(file_name, subsuffix_level=0):
    """
    remove the suffix for a file name
    :param file_name: give me a file name, like: "/home/xuyuxing/file.txt" or "~/work/file.txt" or just "file.txt"
    :param subsuffix_level: sometimes a file can have more than one suffix, like "~/work/file.txt.gz",
           how many you want to remove, 0 meanings remove all suffix
    :return: file without suffix, will keep dir_name as a path, to get base file name: os.path.basename(file_name)
    """

    file_name = os.path.abspath(file_name)
    dir_name = os.path.dirname(file_name)
    base_name = os.path.basename(file_name)
    base_name_split = base_name.split(".")
    if subsuffix_level == 0:
        subsuffix_level = len(base_name_split) - 1
    elif subsuffix_level > len(base_name_split) - 1:
        raise ValueError("There are not enough suffix level to remove")

    return dir_name + "/" + printer_list(base_name.split(".")[:-subsuffix_level], ".")


def md5sum_maker(file_name):
    import hashlib

    if not os.path.exists(file_name):
        FileNotFoundError("No such file or directory: %s" % file_name)

    with open(file_name, "rb") as file_to_check:
        data = file_to_check.read()
        md5_returned = hashlib.md5(data).hexdigest()

    return md5_returned


def cmd_run(cmd_string, cwd=None, retry_max=5, silence=False, log_file=None):
    module_logger = logging_init("cmd_run", log_file)
    module_logger.info("Calling a bash cmd with retry_max %d: %s" %
                       (retry_max, cmd_string))
    if not silence:
        print("Running " + str(retry_max) + " " + cmd_string)
    p = subprocess.Popen(cmd_string, shell=True,
                         stderr=subprocess.PIPE, stdout=subprocess.PIPE, cwd=cwd)
    output, error = p.communicate()
    if not silence:
        print(error.decode())
    returncode = p.poll()
    module_logger.info("Finished bash cmd with returncode as %d" % returncode)
    if returncode == 1:
        if retry_max > 1:
            retry_max = retry_max - 1
            cmd_run(cmd_string, cwd=cwd, retry_max=retry_max)
    del module_logger.handlers[:]

    output = output.decode()
    error = error.decode()

    return (not returncode, output, error)


def multiprocess_running(f, args_list, pool_num, log_file=None, silence=True, args_id_list=None, timeout=None):
    """
    :param f: function name
    :param args_list: a list include every args in a tuple
    :param pool_num: process num of running at sametime
    :param silence:
    :return:

    example

    def f(x, y):
        time.sleep(1)
        return x * y

    args_list = list(zip(range(1,200),range(2,201)))
    pool_num = 5

    multiprocess_running(f, args_list, pool_num, log_file='/lustre/home/xuyuxing/Work/Other/saif/Meth/tmp/log')

    """
    num_tasks = len(list(args_list))

    module_log = logging_init(f.__name__, log_file)
    if not silence:
        print('args_list have %d object and pool_num is %d' %
              (num_tasks, pool_num))
    module_log.info('running with mulitprocess')
    module_log.info('args_list have %d object and pool_num is %d' %
                    (num_tasks, pool_num))

    p_dict = {}

    if len(args_list) > 0:

        if timeout is None:

            f_with_para = partial(get_more_para, f)

            #    def get_more_para(para_tuple):
            #        return f(*para_tuple)

            #    args_list_unziped = zip(*args_list)
            start_time = time.time()
            if not silence:
                print(time_now() + '\tBegin: ')
            module_log.info('Begin: ')
            with Pool(processes=pool_num) as pool:
                # for i, output in enumerate(pool.imap_unordered(f_with_para, args_list, chunksize=1)):
                for i, output in enumerate(pool.imap(f_with_para, args_list, chunksize=1)):
                    if args_id_list:
                        job_id = args_id_list[i]
                    else:
                        job_id = 'ID_' + str(i)

                    p_dict[job_id] = {
                        'args': args_list[i],
                        'output': output,
                        'error': None
                    }
                    # print(i)
                    round_time = time.time()
                    if round_time - start_time > 5:
                        if not silence:
                            print(time_now() + '\t%d/%d %.2f%% parsed' %
                                  (i, num_tasks, i / num_tasks * 100))
                        module_log.info('%d/%d %.2f%% parsed' %
                                        (i, num_tasks, i / num_tasks * 100))
                        start_time = round_time
                    # module_log.info('%d/%d %.2f%% parsed' % (i, num_tasks, i / num_tasks * 100))

            module_log.info('%d/%d %.2f%% parsed' %
                            (i, num_tasks, i / num_tasks * 100))
            module_log.info('All args_list task finished')

            if not silence:
                print(time_now() + '\t%d/%d %.2f%% parsed' %
                      (i, num_tasks, i / num_tasks * 100))
                print(time_now() + '\tAll args_list task finished')

        else:

            f_with_para = partial(get_more_para, f)
            abortable_func = partial(
                abortable_worker, f_with_para, timeout=timeout)

            #    def get_more_para(para_tuple):
            #        return f(*para_tuple)

            #    args_list_unziped = zip(*args_list)

            start_time = time.time()

            if not silence:
                print(time_now() + '\tBegin: ')
            module_log.info('Begin: ')

            with Pool(processes=pool_num, maxtasksperchild=1) as pool:
                # for i, output in enumerate(pool.imap_unordered(f_with_para, args_list, chunksize=1)):
                it = pool.imap(abortable_func, args_list, chunksize=1)
                i = -1
                while 1:
                    i += 1
                    # print(i)

                    try:
                        output = it.next()

                        if args_id_list:
                            job_id = args_id_list[i]
                        else:
                            job_id = 'ID_' + str(i)

                        if output == "Aborting due to timeout":
                            p_dict[job_id] = {
                                'args': args_list[i],
                                'output': None,
                                'error': "timeout"
                            }
                        else:
                            p_dict[job_id] = {
                                'args': args_list[i],
                                'output': output,
                                'error': None
                            }

                        # print(i)
                        round_time = time.time()
                        if round_time - start_time > 5:
                            if not silence:
                                print('%d/%d %.2f%% parsed' %
                                      (i, num_tasks, i / num_tasks * 100))
                            module_log.info('%d/%d %.2f%% parsed' %
                                            (i, num_tasks, i / num_tasks * 100))
                            start_time = round_time

                        # module_log.info('%d/%d %.2f%% parsed' % (i, num_tasks, i / num_tasks * 100))

                    except StopIteration:
                        break

            module_log.info('%d/%d %.2f%% parsed' %
                            (i, num_tasks, i / num_tasks * 100))
            module_log.info('All args_list task finished')

            if not silence:
                print(time_now() + '%d/%d %.2f%% parsed' %
                      (i, num_tasks, i / num_tasks * 100))
                print(time_now() + 'All args_list task finished')

    del module_log.handlers[:]

    return p_dict


def get_more_para(f, para_tuple):
    return f(*para_tuple)


def abortable_worker(func, *args, **kwargs):
    timeout = kwargs.get('timeout', None)
    p = ThreadPool(1)
    res = p.apply_async(func, args=args)
    try:
        out = res.get(timeout)  # Wait timeout seconds for func to complete.
        return out
    except TimeoutError:
        return "Aborting due to timeout"
        raise



from Bio import PDB as pdb
from pybel import readfile, Outputfile


def merge_protein_and_ligand(protein_pdb, ligand_pdb, merge_pdb):
    """
    protein_pdb="/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/vina/FKN_apo.pdb"
    ligand_pdb="/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/vina/FKN_apo_vs_n3_ligand_1.pdb"
    merge_pdb="/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/vina/FKN_with_n3.pdb"
    """

    parser = pdb.PDBParser()
    protein_s = parser.get_structure('FKN', protein_pdb)
    protein_l = parser.get_structure('n3', ligand_pdb)


def pybel_file_covert(input_file, input_type, output_file, output_type):
    """
    check informats and outformats
    """
    input = readfile(input_type, input_file)
    output = Outputfile(output_type, output_file)
    for i in input:
        output.write(i)
    output.close()

if __name__ == '__main__':
    pdbqt_file = '/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/fkn_redo/vina/FKN_apo_vs_n3_ligand_1.pdbqt'
    pdb_file = '/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/fkn_redo/vina/FKN_apo_vs_n3_ligand_1.pdb'

    pybel_file_covert(pdbqt_file, 'pdbqt', pdb_file, 'pdb')


from lib.common.os import mkdir, copy_file, cmd_run, multiprocess_running
import pandas as pd
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.fileIO import tsv_file_dict_parse
from collections import OrderedDict
from lib.common.util import printer_list
from lib.common.util import logging_init


def OG_tsv_file_parse(file_name):
    file_info = tsv_file_dict_parse(file_name)

    OG_dict = OrderedDict()
    for i in file_info:
        OG_id = file_info[i]['Orthogroup']
        OG_dict[OG_id] = {}
        for j in file_info[i]:
            if j == 'Orthogroup':
                continue
            if file_info[i][j]:
                OG_dict[OG_id][j] = file_info[i][j].split(", ")
            else:
                OG_dict[OG_id][j] = ""

    return OG_dict


def write_OG_tsv_file(OG_dict, output_file, species_list=None):
    if not species_list:
        species_list = list(OG_dict[list(OG_dict.keys())[0]].keys())

    with open(output_file, 'w') as f:
        f.write(printer_list(species_list, head='Orthogroup\t')+"\n")

        for OG_id in OG_dict:
            string_list = []
            for sp_id in species_list:
                string_list.append(printer_list(
                    OG_dict[OG_id][sp_id], sep=', '))
            output_string = printer_list(string_list, head=OG_id+"\t")
            f.write(output_string+"\n")

    return output_file


def read_species_info(species_info_table):
    species_info_df = pd.read_excel(species_info_table)
    species_info_dict = {}
    for index in species_info_df.index:
        sp_id = str(species_info_df.loc[index]['sp_id'])
        species_info_dict[sp_id] = {}
        species_info_dict[sp_id]['pt_file'] = str(
            species_info_df.loc[index]['pt_file'])

        if 'cds_file' in species_info_df.loc[index]:
            species_info_dict[sp_id]['cds_file'] = str(
                species_info_df.loc[index]['cds_file'])

    return species_info_dict


def prepare_OG_work_dir(OG_dict, species_info_dict, top_work_dir):
    mkdir(top_work_dir, True)

    # if have cds
    if 'cds_file' in species_info_dict[list(species_info_dict.keys())[0]]:
        cds_flag = True
    else:
        cds_flag = False

    huge_pt_dict = {}
    if cds_flag:
        huge_cds_dict = {}
    huge_rename_map = {}

    # read fasta

    for i in species_info_dict:
        pt_fa_dict = read_fasta_by_faidx(species_info_dict[i]['pt_file'])
        if cds_flag:
            cds_fa_dict = read_fasta_by_faidx(species_info_dict[i]['cds_file'])

        pt_dict = {}
        if cds_flag:
            cds_dict = {}
        rename_map = {}

        num = 0
        for j in pt_fa_dict:
            pt_dict[num] = pt_fa_dict[j].seq
            if cds_flag:
                cds_dict[num] = cds_fa_dict[j].seq
            rename_map[j] = num
            num += 1

        huge_pt_dict[i] = pt_dict
        if cds_flag:
            huge_cds_dict[i] = cds_dict
        huge_rename_map[i] = rename_map

    # write seq to each OG
    for OG_id in OG_dict:
        OG_work = top_work_dir + "/" + OG_id
        mkdir(OG_work, True)

        # pt_file
        pt_fa = OG_work + "/pt.fa"
        with open(pt_fa, 'w') as f:
            for sp_id in OG_dict[OG_id]:
                for g_id in OG_dict[OG_id][sp_id]:
                    if g_id != '':
                        pt_seq = huge_pt_dict[sp_id][huge_rename_map[sp_id][g_id]]
                        f.write(">%s\n%s\n" % (
                            str(huge_rename_map[sp_id][g_id])+"_"+sp_id, pt_seq))

        # cds_file
        if cds_flag:
            cds_fa = OG_work + "/cds.fa"
            with open(cds_fa, 'w') as f:
                for sp_id in OG_dict[OG_id]:
                    for g_id in OG_dict[OG_id][sp_id]:
                        if g_id != '':
                            cds_seq = huge_cds_dict[sp_id][huge_rename_map[sp_id][g_id]]
                            f.write(">%s\n%s\n" % (
                                str(huge_rename_map[sp_id][g_id])+"_"+sp_id, cds_seq))

        # rename_map
        rename_map = OG_work + "/rename.map"
        with open(rename_map, 'w') as f:
            for sp_id in OG_dict[OG_id]:
                for g_id in OG_dict[OG_id][sp_id]:
                    if g_id != '':
                        f.write("%s\t%s\t%s\n" %
                                (huge_rename_map[sp_id][g_id], g_id, sp_id))

    # write rename file
    all_rename_map = top_work_dir + "/rename.map"
    with open(all_rename_map, 'w') as f:
        for sp_id in huge_rename_map:
            for g_id in huge_rename_map[sp_id]:
                if g_id != '':
                    f.write("%s\t%s\t%s\n" %
                            (huge_rename_map[sp_id][g_id], g_id, sp_id))


def fasttree_one(OG_work_dir):
    pt_fa = OG_work_dir + "/pt.fa"

    aa_aln_file = pt_fa+".aln"
    aa_dnd_file = pt_fa+".dnd"
    #cmd_string = "t_coffee "+aa_file+" -method mafftgins_msa muscle_msa kalign_msa t_coffee_msa -output=fasta_aln -outfile="+aa_aln_file+" -newtree "+aa_dnd_file
    cmd_string = "clustalw2 -INFILE="+pt_fa + \
        " -ALIGN -OUTPUT=FASTA -OUTFILE="+aa_aln_file+" -type=protein"
    cmd_run(cmd_string, silence=True)

    tree_file = OG_work_dir+"/fasttree.phb"
    cmd_string = "FastTree -wag -gamma -out %s %s >/dev/null" % (
        tree_file, aa_aln_file)
    cmd_run(cmd_string, silence=True)

    return pt_fa, aa_aln_file, tree_file


def fasttree_many(OG_tsv_file, species_info_dict, work_dir, num_threads):
    mkdir(work_dir, True)
    log_file = work_dir + "/log"

    module_log = logging_init("FastTree pipeline", log_file)

    # load orthofinder group
    module_log.info('load orthofinder group')
    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    # prepare OG work dir
    module_log.info('prepare OG work dir')
    prepare_OG_work_dir(OG_dict, species_info_dict, work_dir)

    # running treebest
    module_log.info('running fasttree')
    args_list = []

    for OG_id in OG_dict:
        OG_work_dir = work_dir + "/" + OG_id
        args_list.append((OG_work_dir,))

    multiprocess_running(fasttree_one, args_list,
                         num_threads, log_file=log_file)

    return work_dir


def orthofinder_running(work_dir, species_info_dict, sp_list=None, only_on_orthogroup=True, only_print=False):
    of_dir = work_dir
    of_pt_dir = of_dir + "/pt_file"
    mkdir(of_dir, True)
    mkdir(of_pt_dir, True)

    if not sp_list:
        sp_list = list(species_info_dict.keys())

    for sp_id in sp_list:
        pt_file = species_info_dict[sp_id]['pt_file']
        copy_file(pt_file, of_pt_dir+"/%s.fasta" % sp_id)

    if only_print:
        print("source activate orthofinder")
        print("cd %s" % of_dir)
        if only_on_orthogroup:
            print("qx -pe smp 56 orthofinder -f pt_file -t 56 -a 56 -S diamond -og")


if __name__ == '__main__':
    ref_xlsx = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/Gel_ref.xlsx'
    sp_list = ["Ini", "Cau", "Sly", "Mgu", "Cca", "Ath", "Aco", "Gel", "Vpl", "Ash", "Aof", "Osa", "Acom", "Cnu", "Eve", "Xvi", "Pni", "Atr"]
    orthofinder_dir = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline'

    species_info_dict = read_species_info(ref_xlsx)
    species_info_dict = {i:species_info_dict[i] for i in species_info_dict if i in sp_list}

    orthofinder_running(orthofinder_dir, species_info_dict, sp_list=None, only_on_orthogroup=True, only_print=True)

    #
    OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline/pt_file/OrthoFinder/Results_Apr13/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline/fasttree"

    fasttree_many(OG_tsv_file, species_info_dict, fasttree_dir, 160)


    # for canrong

    ref_xlsx = '/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/seq_info.xlsx'
    sp_list = ["Osa", "Zma", "Acom", "Xvi", "Aco", "Ath", "Mdo", "Sly", "Pni", "Atr"]

    species_info_dict = read_species_info(ref_xlsx)
    species_info_dict = {i:species_info_dict[i] for i in species_info_dict if i in sp_list}

    #
    OG_tsv_file = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/pt_file/OrthoFinder/Results_May08/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/fasttree"

    fasttree_many(OG_tsv_file, species_info_dict, fasttree_dir, 56)



from interlap import InterLap
from collections import OrderedDict

__author__ = 'Yuxing Xu'


def section(inter_a, inter_b, int_flag=False, just_judgement=False):
    """
    get the section
    :param inter_a:
    :param inter_b:
    :return:
    """
    all = sorted(list(inter_a) + list(inter_b))
    deta = (all[1], all[2])
    if int_flag is False:
        if max(inter_a) >= min(inter_b) and max(inter_b) >= min(inter_a):
            If_inter = True  # Yes
        else:
            If_inter = False  # No

        if just_judgement:
            return If_inter
        else:
            return If_inter, deta

    else:
        if max(inter_a) - min(inter_b) >= -1 and max(inter_b) - min(inter_a) >= -1:
            If_inter = True  # Yes
        else:
            If_inter = False  # No

        if just_judgement:
            return If_inter
        else:
            return If_inter, deta


def merge_intervals(input_list, int=False):
    """
    a function that will merge overlapping intervals
    :param intervals: a list of tuples
                      e.g. intervals = [(1,5),(33,35),(40,33),(10,15),(13,18),(28,23),(70,80),(22,25),(38,50),(40,60)]
    :param int: if the data is all int
    :return: merged list
    """
    intervals = []
    for i in input_list:
        intervals.append(tuple(i))

    sorted_by_lower_bound = sorted(intervals, key=lambda tup: min(tup))
    merged = []

    for higher in sorted_by_lower_bound:
        if not merged:
            merged.append(higher)
        else:
            lower = merged[-1]
            # test for intersection between lower and higher:
            # we know via sorting that lower[0] <= higher[0]
            if int is False:
                if min(higher) <= max(lower):
                    upper_bound = max(lower + higher)
                    # replace by merged interval
                    merged[-1] = (min(lower), upper_bound)
                else:
                    merged.append(higher)
            elif int is True:
                if min(higher) <= max(lower):
                    upper_bound = max(lower + higher)
                    # replace by merged interval
                    merged[-1] = (min(lower), upper_bound)
                elif max(lower) + 1 == min(higher):
                    upper_bound = max(lower + higher)
                    # replace by merged interval
                    merged[-1] = (min(lower), upper_bound)
                else:
                    merged.append(higher)
    return merged


def overturn(inter_list):
    """
    input a list of intervals and the function will overturn it to a list with gap of the intervals
    :param inter_list:
    :return: gap_list
    """
    inter_list = sorted(merge_intervals(inter_list, True))
    output_list = []
    last_right = 0
    for index in range(0, len(inter_list) + 1):
        if index == 0:
            output_list.append((float('-inf'), inter_list[index][0] - 1))
            last_right = inter_list[index][1]
        elif index == len(inter_list):
            output_list.append((last_right + 1, float('inf')))
        else:
            output_list.append((last_right + 1, inter_list[index][0] - 1))
            last_right = inter_list[index][1]
    return output_list


def interval_minus_set(target, bullets):
    if len(bullets) == 0:
        return [target]
    gaps = overturn(bullets)
    output_list = []
    for i in gaps:
        If_inter, deta = section(target, i)
        if If_inter:
            output_list.append(deta)
    return output_list


def overlap_between_interval_set(interval1, interval2, similarity_type='shorter_overlap_coverage'):
    """
    used for compare two interval_set

    similarity_type can be: "shorter_overlap_coverage" or "jaccord_score"

    interval1 = [(1,20), (50,60), (80,100)]
    interval2 = [(50,70), (90,100)]
    """

    interval1 = merge_intervals(interval1)
    interval2 = merge_intervals(interval2)

    interval2_interLap = InterLap(interval2)

    overlap = []
    for it1 in interval1:
        for it2 in interval2_interLap.find(it1):
            flag, deta = section(it1, it2, True)
            if flag:
                overlap.append(deta)

    overlap = merge_intervals(overlap)

    overlap_length = sum_interval_length(overlap)
    if similarity_type == 'shorter_overlap_coverage':
        shorter_len = min(sum_interval_length(interval1),
                          sum_interval_length(interval2))
        if shorter_len > 0:
            overlap_ratio = overlap_length/shorter_len
        else:
            overlap_ratio = 0

    elif similarity_type == 'jaccord_score':
        overlap_ratio = overlap_length / \
            sum_interval_length(merge_intervals(interval1 + interval2))

    return overlap_ratio, overlap_length, overlap


def group_by_intervals(list_input, int=False):
    """
    :param list_input: [(ID1,start,end),(ID2,start,end),(ID3,start,end),(ID4,start,end)]
    :return: dict = {
                    "group1" : {
                                "range" : (start,end)
                                "list" : [ID1,ID2]
                                }
                    "group2" : {
                                "range" : (start,end)
                                "list" : [ID3,ID4]
                                }
                    }
    """
    range_dict = {}
    intervals = []

    for ID, start, end in list_input:
        range_key = (min(start, end), max(start, end))
        if not range_key in range_dict:
            range_dict[range_key] = []
        range_dict[range_key].append(ID)
        intervals.append(range_key)

    intervals = list(set(intervals))

    sorted_by_lower_bound = sorted(intervals, key=lambda tup: min(tup))
    merged = []
    merged_ID = {}
    for higher in sorted_by_lower_bound:
        if not merged:
            merged.append(higher)
            merged_ID[higher] = range_dict[higher]
        else:
            lower = merged[-1]
            # test for intersection between lower and higher:
            # we know via sorting that lower[0] <= higher[0]
            if int is False:
                if min(higher) <= max(lower):
                    upper_bound = max(lower + higher)
                    later_ID = merged_ID[merged[-1]]
                    del merged_ID[merged[-1]]
                    merged_ID[(min(lower), upper_bound)] = later_ID + \
                        list(range_dict[higher])
                    # replace by merged interval
                    merged[-1] = (min(lower), upper_bound)
                else:
                    merged.append(higher)
                    merged_ID[higher] = range_dict[higher]
            elif int is True:
                if min(higher) <= max(lower):
                    upper_bound = max(lower + higher)
                    later_ID = merged_ID[merged[-1]]
                    del merged_ID[merged[-1]]
                    merged_ID[(min(lower), upper_bound)] = later_ID + \
                        list(range_dict[higher])
                    # replace by merged interval
                    merged[-1] = (min(lower), upper_bound)
                elif max(lower) + 1 == min(higher):
                    upper_bound = max(lower + higher)
                    later_ID = merged_ID[merged[-1]]
                    del merged_ID[merged[-1]]
                    merged_ID[(min(lower), upper_bound)] = later_ID + \
                        list(range_dict[higher])
                    # replace by merged interval
                    merged[-1] = (min(lower), upper_bound)
                else:
                    merged.append(higher)
                    merged_ID[higher] = range_dict[higher]
    return merged_ID


def group_by_intervals_with_overlap_threshold(input_dir, overlap_threshold=0.5):
    """
    :param input_dir =
                    {
                    "ID1" : (start,end),
                    "ID2" : (start,end),
                    "ID3" : (start,end),
                    "ID4" : (start,end)
                    }
            input_dir = {
                "A": (100, 500),
                "B": (99, 501),
                "C": (480, 1000),
                "D": (500, 1005),
                "E": (80, 600),
                "F": (600, 1100),
                "G": (2000, 2500),
                "H": (1900, 2600),
                "I": (2100, 2600),
                "J": (430, 500),
                "K": (100, 500),
                "L": (600, 1000),
            }


    :return: dict = {
                    "group1" : {
                                "range" : (start,end)
                                "list" : [ID1,ID2]
                                }
                    "group2" : {
                                "range" : (start,end)
                                "list" : [ID3,ID4]
                                }
                    }
    """

    def higer_in_lower_group(merged_group, lower_group_id, higher, overlap_threshold):
        grouped_flag = 0
        for i in merged_group[lower_group_id]:
            ifflag, deta = section(higher, i)
            if ifflag:
                len_h = max(higher[0], higher[1]) - \
                    min(higher[0], higher[1]) + 1
                len_i = max(i[0], i[1]) - min(i[0], i[1]) + 1
                len_d = max(deta[0], deta[1]) - min(deta[0], deta[1]) + 1
                if len_d / len_h > overlap_threshold or len_d / len_i > overlap_threshold:
                    grouped_flag = 1
                    break
        return grouped_flag

    range_dict = {}
    intervals = []
    for ID in input_dir:
        start, end = input_dir[ID]
        range_key = (min(start, end), max(start, end))
        if not range_key in range_dict:
            range_dict[range_key] = []
        range_dict[range_key].append(ID)
        intervals.append(range_key)
    intervals = list(set(intervals))

    sorted_by_lower_bound = sorted(intervals, key=lambda tup: min(tup))
    merged_group = OrderedDict()
    group_id = 0
    for higher in sorted_by_lower_bound:
        if len(merged_group) == 0:
            group_id = group_id + 1
            merged_group["group_%d" % group_id] = [higher]
        else:
            lower_group_id = list(merged_group.keys())[-1]
            # test for intersection between lower and higher:
            # we know via sorting that lower[0] <= higher[0]
            if higer_in_lower_group(merged_group, lower_group_id, higher, overlap_threshold):
                merged_group["group_%d" % group_id].append(higher)
            else:
                group_id = group_id + 1
                merged_group["group_%d" % group_id] = [higher]

    for group_id in merged_group:
        range_list = merged_group[group_id]
        input_id_list = []
        for i in range_list:
            input_id_list.extend(range_dict[i])
        merged_group[group_id] = {
            "range": merge_intervals(range_list),
            "list": input_id_list
        }

    return merged_group


def sum_interval_length(range_list, close_range=True):
    length_sum = 0
    for i in range_list:
        if close_range:
            length_sum = length_sum + max(i) - min(i) + 1
        else:
            length_sum = length_sum + max(i) - min(i)
    return length_sum


# -*- coding: utf-8 -*-
# #START_LICENSE###########################################################
#
#
# This file is part of the Environment for Tree Exploration program
# (ETE).  http://ete.cgenomics.org
#  
# ETE is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#  
# ETE is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#  
# You should have received a copy of the GNU General Public License
# along with ETE.  If not, see <http://www.gnu.org/licenses/>.
#
# 
#                     ABOUT THE ETE PACKAGE
#                     =====================
# 
# ETE is distributed under the GPL copyleft license (2008-2011).  
#
# If you make use of ETE in published work, please cite:
#
# Jaime Huerta-Cepas, Joaquin Dopazo and Toni Gabaldon.
# ETE: a python Environment for Tree Exploration. Jaime BMC
# Bioinformatics 2010,:24doi:10.1186/1471-2105-11-24
#
# Note that extra references to the specific methods implemented in 
# the toolkit are available in the documentation. 
# 
# More info at http://ete.cgenomics.org
#
# 
# #END_LICENSE#############################################################
__VERSION__="ete2-2.2rev1026" 
#START_LICENSE###########################################################
#
# Copyright (C) 2009 by Jaime Huerta Cepas. All rights reserved.
# email: jhcepas@gmail.com
#
# This file is part of the Environment for Tree Exploration program (ETE).
# http://ete.cgenomics.org
#
# ETE is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# ETE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ETE.  If not, see <http://www.gnu.org/licenses/>.
#
# #END_LICENSE#############################################################
import os
try:
    import cPickle as pickle
except ImportError:
    import pickle
import random
import copy
from collections import deque 
import itertools
import functools
from .newick import read_newick, write_newick

TREEVIEW = False
__all__ = ["Tree", "TreeNode"]

DEFAULT_COMPACT = False
DEFAULT_SHOWINTERNAL = False
DEFAULT_DIST = 1.0
DEFAULT_SUPPORT = 1.0
DEFAULT_NAME = "NoName"

class TreeError(Exception):
    """
    A problem occurred during a TreeNode operation
    """
    def __init__(self, value=''):
        self.value = value
    def __str__(self):
        return repr(self.value)

class TreeNode(object):
    """ 
    TreeNode (Tree) class is used to store a tree structure. A tree
    consists of a collection of TreeNode instances connected in a
    hierarchical way. Trees can be loaded from the New Hampshire Newick
    format (newick).
    :argument newick: Path to the file containing the tree or, alternatively,
       the text string containing the same information.
    :argument 0 format: subnewick format 
      .. table::                                               
          ======  ============================================== 
          FORMAT  DESCRIPTION                                    
          ======  ============================================== 
          0        flexible with support values                  
          1        flexible with internal node names             
          2        all branches + leaf names + internal supports 
          3        all branches + all names                      
          4        leaf branches + leaf names                    
          5        internal and leaf branches + leaf names       
          6        internal branches + leaf names                
          7        leaf branches + all names                     
          8        all names                                     
          9        leaf names                                    
          100      topology only                                 
          ======  ============================================== 
    :returns: a tree node object which represents the base of the tree.
    ** Examples: **
    :: 
        t1 = Tree() # creates an empty tree
        t2 = Tree('(A:1,(B:1,(C:1,D:1):0.5):0.5);')
        t3 = Tree('/home/user/myNewickFile.txt')
    """

    def _get_dist(self):
        return self._dist
    def _set_dist(self, value):
        try:
            self._dist = float(value)
        except ValueError:
            raise

    def _get_support(self):
        return self._support
    def _set_support(self, value):
        try:
            self._support = float(value)
        except ValueError:
            raise

    def _get_up(self):
        return self._up
    def _set_up(self, value):
        if type(value) == type(self) or value is None:
            self._up = value
        else:
            raise ValueError("bad node_up type")

    def _get_children(self):
        return self._children
    def _set_children(self, value):
        if type(value) == list and \
           len(set([type(n)==type(self) for n in value]))<2:
            self._children = value
        else:
            raise ValueError("bad children type")

    def _get_style(self):
        if self._img_style is None:
            self._set_style(None)
           
        return self._img_style

    def _set_style(self, value):
        self.set_style(value)
            
    #: Branch length distance to parent node. Default = 0.0
    img_style = property(fget=_get_style, fset=_set_style)
             
    #: Branch length distance to parent node. Default = 0.0
    dist = property(fget=_get_dist, fset=_set_dist)
    #: Branch support for current node
    support = property(fget=_get_support, fset=_set_support)
    #: Pointer to parent node
    up = property(fget=_get_up, fset=_set_up)
    #: A list of children nodes
    children = property(fget=_get_children, fset=_set_children)

    def _set_face_areas(self, value):
        if isinstance(value, _FaceAreas):
            self._faces = value
        else:
            raise ValueError("[%s] is not a valid FaceAreas instance" %type(value))
        
    def _get_face_areas(self):
        if not hasattr(self, "_faces"):
            self._faces = _FaceAreas()
        return self._faces

    faces = property(fget=_get_face_areas, \
                         fset=_set_face_areas)

    def __init__(self, newick=None, format=0, dist=None, support=None,
                 name=None):
        self._children = []
        self._up = None
        self._dist = DEFAULT_DIST
        self._support = DEFAULT_SUPPORT
        self._img_style = None
        self.features = set([])
        # Add basic features
        self.features.update(["dist", "support", "name"])
        if dist is not None:
            self.dist = dist
        if support is not None:
            self.support = support

        self.name = name if name is not None else DEFAULT_NAME
        
        # Initialize tree
        if newick is not None:
            read_newick(newick, root_node = self, format=format)
           

    def __bool__(self):
        return True
    
    __nonzero__ = __bool__

    def __repr__(self):
        return "Tree node '%s' (%s)" %(self.name, hex(self.__hash__()))

    def __and__(self, value):
        """ This allows to execute tree&'A' to obtain the descendant node
        whose name is A"""
        value=str(value)
        try:
            first_match = next(self.iter_search_nodes(name=value))
            return first_match
        except StopIteration:
            raise ValueError("Node not found")

    def __add__(self, value):
        """ This allows to sum two trees."""
        # Should a make the sum with two copies of the original trees?
        if type(value) == self.__class__:
            new_root = self.__class__()
            new_root.add_child(self)
            new_root.add_child(value)
            return new_root
        else:
            raise ValueError("Invalid node type")

    def __str__(self):
        """ Print tree in newick format. """
        return self.get_ascii(compact=DEFAULT_COMPACT, \
                                show_internal=DEFAULT_SHOWINTERNAL)

    def __contains__(self, item):
        """ Check if item belongs to this node. The 'item' argument must
        be a node instance or its associated name."""
        if isinstance(item, self.__class__):
            return item in set(self.get_descendants())
        elif type(item)==str:
            return item in set([n.name for n in self.traverse()])

    def __len__(self):
        """Node len returns number of children."""
        return len(self.get_leaves())

    def __iter__(self):
        """ Iterator over leaf nodes"""
        return self.iter_leaves()

    def add_feature(self, pr_name, pr_value):
        """ 
        Add or update a node's feature. 
        """
        setattr(self, pr_name, pr_value)
        self.features.add(pr_name)

    def add_features(self, **features):
        """ 
        Add or update several features. """
        for fname, fvalue in features.items():
            setattr(self, fname, fvalue)
            self.features.add(fname)

    def del_feature(self, pr_name):
        """ 
        Permanently deletes a node's feature. 
        """
        if hasattr(self, pr_name):
            delattr(self, pr_name)
            self.features.remove(pr_name)

    # Topology management
    def add_child(self, child=None, name=None, dist=None, support=None):
        """
        Adds a new child to this node. If child node is not suplied
        as an argument, a new node instance will be created.
        :argument None child: the node instance to be added as a child.
        :argument None name: the name that will be given to the child.
        :argument None dist: the distance from the node to the child.
        :argument None support': the support value of child partition.
        :returns: The child node instance
        """
        if child is None:
            child = self.__class__()
        
        if name is not None:
            child.name = name
        if dist is not None:
            child.dist = dist
        if support is not None:
            child.support = support
            
        self.children.append(child)
        child.up = self
        return child

    def remove_child(self, child):
        """ 
        Removes a child from this node (parent and child
        nodes still exit but are no longer connected). 
        """
        try:
            self.children.remove(child)
        except ValueError as e:
            raise TreeError(e)
        else:
            child.up = None
            return child

    def add_sister(self, sister=None, name=None, dist=None):
        """
        Adds a sister to this node. If sister node is not supplied
        as an argument, a new TreeNode instance will be created and
        returned.
        """
        if self.up == None:
            raise TreeError("A parent node is required to add a sister")
        else:
            return self.up.add_child(child=sister, name=name, dist=dist)

    def remove_sister(self, sister=None):
        """
        Removes a node's sister node. It has the same effect as
        **`TreeNode.up.remove_child(sister)`**
        If a sister node is not supplied, the first sister will be deleted
        and returned.
        :argument sister: A node instance
        :return: The node removed
        """
        sisters = self.get_sisters()
        if len(sisters)>0:
            if sister==None:
                sister = sisters.pop(0)
            return self.up.remove_child(sister)

    def delete(self, prevent_nondicotomic=True, preserve_branch_length=False):
        """
        Deletes node from the tree structure. Notice that this method
        makes 'disappear' the node from the tree structure. This means
        that children from the deleted node are transferred to the
        next available parent.
        :param True prevent_nondicotomic: When True (default), delete
        function will be execute recursively to prevent single-child
        nodes.
        :param False preserve_branch_length: If True, branch lengths
        of the deleted nodes are transferred (summed up) to its
        parent's branch, thus keeping original distances among nodes.
                
        **Example:**
        ::
                / C
          root-|
               |        / B
                \--- H |
                        \ A
          > root.delete(H) will produce this structure:
                / C
               |
          root-|--B
               |
                \ A
        """
        parent = self.up
        if parent:
            for ch in self.children:
                if preserve_branch_length:
                    ch.dist += self.dist
                parent.add_child(ch)

            parent.remove_child(self)

        # Avoids parents with only one child
        if prevent_nondicotomic and parent and\
              len(parent.children)<2:
            parent.delete(prevent_nondicotomic=False,
                          preserve_branch_length=preserve_branch_length)


    def detach(self):
        """
        Detachs this node (and all its descendants) from its parent
        and returns the referent to itself.
        Detached node conserves all its structure of descendants, and can
        be attached to another node through the 'add_child' function. This
        mechanism can be seen as a cut and paste.
        """

        if self.up:
            self.up.children.remove(self)
            self.up = None
        return self


    def prune(self, nodes, preserve_branch_length=False):
        """Prunes the topology of a node in order to conserve only a
        selected list of leaf or internal nodes. The minimum number of
        internal nodes (the deepest as possible) are kept to conserve
        the topological relationship among the provided list of nodes.
        :var nodes: a list of node names or node objects that must be kept
        
        :param False preserve_branch_length: If True, branch lengths
        of the deleted nodes are transferred (summed up) to its
        parent's branch, thus keeping original distances among nodes.
        
        **Examples:**
        ::
          t = Tree("(((A:0.1, B:0.01):0.001, C:0.0001):1.0[&&NHX:name=I], (D:0.00001):0.000001[&&NHX:name=J]):2.0[&&NHX:name=root];")
          node_C = t.search_nodes(name="C")[0]
          t.prune(["A","D", node_C])
          print t
        """
        def cmp_nodes(x, y):
            # if several nodes are in the same path of two kept nodes,
            # only one should be maintained. This prioritize internal
            # nodes that are already in the to_keep list and then
            # deeper nodes (closer to the leaves). 
            if x in to_keep:
                if y not in to_keep:
                    return -1
                elif y in to_keep:
                    return 0
            elif n2depth[x] > n2depth[y]:
                return -1
            elif n2depth[x] < n2depth[y]:
                return 1
            else:
                return 0

        to_keep = set(_translate_nodes(self, *nodes))
        start, node2path = self.get_common_ancestor(to_keep, get_path=True)

        # Calculate which kept nodes are visiting the same nodes in
        # their path to the common ancestor.
        n2count = {}
        n2depth = {}
        for seed, path in node2path.items():
            for visited_node in path: 
                if visited_node not in n2depth:
                    depth = visited_node.get_distance(start, topology_only=True)
                    n2depth[visited_node] = depth
                if visited_node is not seed:
                    n2count.setdefault(visited_node, set()).add(seed)

        # if several internal nodes are in the path of exactly the
        # same kept nodes, only one should be maintain. 
        visitors2nodes = {}
        for node, visitors in n2count.items():
            # keep nodes connection at least two other nodes
            if len(visitors)>1: 
                visitor_key = frozenset(visitors)
                visitors2nodes.setdefault(visitor_key, set()).add(node)
        for visitors, nodes in visitors2nodes.items():
            s = sorted(nodes, key=functools.cmp_to_key(cmp_nodes))
            to_keep.add(s[0])

        # Detach unvisited branches
        if start is not self:
            start.detach()
            for n in self.get_children(): 
                n.detach()
            for n in start.get_children():
                self.add_child(child=n)

        for n in [self]+self.get_descendants():
            if n not in to_keep: 
                n.delete(prevent_nondicotomic=False,
                         preserve_branch_length=preserve_branch_length)

    def swap_children(self):
        """
        Swaps current children order.
        """
        if len(self.children)>1:
            self.children.reverse()

    # def prune_OLD(self, nodes):
    #     """
    #     Prunes the topology of this node in order to conserve only a
    #     selected list of leaf or internal nodes. The algorithm deletes
    #     nodes until getting a consistent topology with a subset of
    #     nodes. Topology relationships among kept nodes is maintained.
    #  
    #     :var nodes: a list of node names or node objects that must be kept
    #  
    #     **Examples:**
    #  
    #     ::
    #  
    #       t = Tree("(((A:0.1, B:0.01):0.001, C:0.0001):1.0[&&NHX:name=I], (D:0.00001):0.000001[&&NHX:name=J]):2.0[&&NHX:name=root];")
    #       node_C = t.search_nodes(name="C")[0]
    #       t.prune(["A","D", node_C])
    #       print t
    #     """
    #    
    #     to_keep = set(_translate_nodes(self, *nodes))
    #     to_detach = []
    #     for node in self.traverse("postorder"):
    #         for c in node.children:
    #             if c in to_keep:
    #                 to_keep.add(node)
    #                 break
    #         if node not in to_keep:
    #             to_detach.append(node)
    #             for c in node.children:
    #                 to_detach.remove(c)
    #     for node in to_detach:
    #         node.detach()
    #     for node in to_keep:
    #         if len(node.children) == 1:
    #             node.delete()
    #     if len(self.children)==1 and self.children[0] not in to_keep:
    #         self.children[0].delete()

    # #####################
    # Tree traversing 
    # #####################


    def get_children(self):
        """ 
        Returns an independent list of node's children. 
        """
        return [ch for ch in self.children]

    def get_sisters(self):
        """ 
        Returns an indepent list of sister nodes. 
        """
        if self.up!=None:
            return [ch for ch in self.up.children if ch!=self]
        else:
            return []

    def iter_leaves(self, is_leaf_fn=None):
        """ 
        Returns an iterator over the leaves under this node. 
        :argument None is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
        """
        for n in self.traverse(strategy="preorder", is_leaf_fn=is_leaf_fn):
            if not is_leaf_fn:
                if n.is_leaf():
                    yield n
            else:
                if is_leaf_fn(n):
                    yield n
                    
    def get_leaves(self, is_leaf_fn=None):
        """
        Returns the list of terminal nodes (leaves) under this node.
        :argument None is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
        """
        return [n for n in self.iter_leaves(is_leaf_fn=is_leaf_fn)]

    def iter_leaf_names(self, is_leaf_fn=None):
        """ 
        Returns an iterator over the leaf names under this node. 
        :argument None is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
        """
        for n in self.iter_leaves(is_leaf_fn=is_leaf_fn):
            yield n.name

    def get_leaf_names(self, is_leaf_fn=None):
        """
        Returns the list of terminal node names under the current
        node.
        :argument None is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
        """
        return [name for name in self.iter_leaf_names(is_leaf_fn=is_leaf_fn)]

    def iter_descendants(self, strategy="levelorder", is_leaf_fn=None):
        """ 
        Returns an iterator over all descendant nodes. 
        :argument None is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
        """
        for n in self.traverse(strategy=strategy, is_leaf_fn=is_leaf_fn):
            if n is not self:
                yield n

    def get_descendants(self, strategy="levelorder", is_leaf_fn=None):
        """
        Returns a list of all (leaves and internal) descendant nodes.
        :argument None is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
        """
        return [n for n in self.iter_descendants(strategy=strategy, \
                                                 is_leaf_fn=is_leaf_fn)]

    def traverse(self, strategy="levelorder", is_leaf_fn=None):
        """
        Returns an iterator to traverse the tree structure under this
        node.
         
        :argument "levelorder" strategy: set the way in which tree
           will be traversed. Possible values are: "preorder" (first
           parent and then children) 'postorder' (first children and
           the parent) and "levelorder" (nodes are visited in order
           from root to leaves)
        :argument None is_leaf_fn: If supplied, ``is_leaf_fn``
           function will be used to interrogate nodes about if they
           are terminal or internal. ``is_leaf_fn`` function should
           receive a node instance as first argument and return True
           or False. Use this argument to traverse a tree by
           dynamically collapsing internal nodes matching
           ``is_leaf_fn``.
        """
        if strategy=="preorder":
            return self._iter_descendants_preorder(is_leaf_fn=is_leaf_fn)
        elif strategy=="levelorder":
            return self._iter_descendants_levelorder(is_leaf_fn=is_leaf_fn)
        elif strategy=="postorder":
            return self._iter_descendants_postorder(is_leaf_fn=is_leaf_fn)
            
    def _iter_descendants_postorder_recursive(self, is_leaf_fn=None):
        """
        Iterate over all desdecendant nodes.
        """
        if not is_leaf_fn or not is_leaf_fn(self):
            for ch in self.children:
                for node in ch._iter_descendants_postorder(is_leaf_fn=is_leaf_fn):
                    yield node
        yield self
    
    def iter_prepostorder(self, is_leaf_fn=None):
        """
        Iterate over all nodes in a tree yielding every node in both
        pre and post order. Each iteration returns a postorder flag
        (True if node is being visited in postorder) and a node
        instance.
        """
        to_visit = [self]
        if is_leaf_fn is not None:
            _leaf = is_leaf_fn
        else:
            _leaf = self.__class__.is_leaf

        while to_visit:
            node = to_visit.pop(-1)
            try:
                node = node[1]
            except TypeError:
                # PREORDER ACTIONS
                yield (False, node)
                if not _leaf(node):
                    # ADD CHILDREN
                    to_visit.extend(reversed(node.children + [[1, node]]))
            else:
                #POSTORDER ACTIONS
                yield (True, node)

    def _iter_descendants_postorder(self, is_leaf_fn=None):
        to_visit = [self]
        if is_leaf_fn is not None:
            _leaf = is_leaf_fn
        else:
            _leaf = self.__class__.is_leaf

        while to_visit:
            node = to_visit.pop(-1)
            try:
                node = node[1]
            except TypeError:
                # PREORDER ACTIONS
                if not _leaf(node):
                    # ADD CHILDREN
                    to_visit.extend(reversed(node.children + [[1, node]]))
                else:
                    yield node
            else:
                #POSTORDER ACTIONS
                yield node
        
    def _iter_descendants_levelorder(self, is_leaf_fn=None):
        """ 
        Iterate over all desdecendant nodes. 
        """
        tovisit = deque([self])
        while len(tovisit)>0:
            node = tovisit.popleft()
            yield node
            if not is_leaf_fn or not is_leaf_fn(node):
                tovisit.extend(node.children)

    def _iter_descendants_preorder(self, is_leaf_fn=None):
        """
        Iterator over all descendant nodes. 
        """
        to_visit = deque()
        node = self
        while node is not None:
            yield node
            if not is_leaf_fn or not is_leaf_fn(node):
                to_visit.extendleft(reversed(node.children))
            try:
                node = to_visit.popleft()
            except:
                node = None

    def iter_ancestors(self):
        '''versionadded: 2.2
        
        Iterates over the list of all ancestor nodes from current node
        to the current tree root.
        '''
        node = self
        while node.up is not None:
            yield node.up
            node = node.up

    def get_ancestors(self):
        '''versionadded: 2.2
        Returns the list of all ancestor nodes from current node to
        the current tree root.
        '''
        return [n for n in self.iter_ancestors()]
            
    def describe(self):
        """ 
        Prints general information about this node and its
        connections.
        """
        if len(self.get_tree_root().children)==2:
            rooting = "Yes"
        elif len(self.get_tree_root().children)>2:
            rooting = "No"
        else:
            rooting = "Unknown"
        max_node, max_dist = self.get_farthest_leaf()
        cached_content = self.get_cached_content()
        print("Number of leaf nodes:\t%d" % len(cached_content[self]))
        print("Number of internal nodes:\t%d" % len(cached_content))
        print("Rooted:\t%s" %rooting)
        print("Most distant node:\t%s" %max_node.name)
        print("Max. distance:\t%f" %max_dist)
        
    def write(self, features=None, outfile=None, format=0, is_leaf_fn=None,
              format_root_node=False):
        """ 
        Returns the newick representation of current node. Several
        arguments control the way in which extra data is shown for
        every node:
        :argument features: a list of feature names to be exported
          using the Extended Newick Format (i.e. features=["name",
          "dist"]). Use an empty list to export all available features
          in each node (features=[])
        :argument outfile: writes the output to a given file
        :argument format: defines the newick standard used to encode the
          tree. See tutorial for details.
        :argument False format_root_node: If True, it allows features
          and branch information from root node to be exported as a
          part of the newick text string. For newick compatibility
          reasons, this is False by default.
        :argument is_leaf_fn: See :func:`TreeNode.traverse` for
          documentation.
          
        **Example:**
        ::
             t.get_newick(features=["species","name"], format=1)
        """

        nw = write_newick(self, features = features, format=format,
                          is_leaf_fn=is_leaf_fn,
                          format_root_node=format_root_node)
        if outfile is not None:
            open(outfile, "w").write(nw)
        else:
            return nw

    def get_tree_root(self):
        """ 
        Returns the absolute root node of current tree structure. 
        """
        root = self
        while root.up is not None:
            root = root.up
        return root

    def get_common_ancestor(self, *target_nodes, **kargs):
        """ 
        Returns the first common ancestor between this node and a given
        list of 'target_nodes'.
        **Examples:**
        ::
          t = tree.Tree("(((A:0.1, B:0.01):0.001, C:0.0001):1.0[&&NHX:name=common], (D:0.00001):0.000001):2.0[&&NHX:name=root];")
          A = t.get_descendants_by_name("A")[0]
          C = t.get_descendants_by_name("C")[0]
          common =  A.get_common_ancestor(C)
          print common.name
        """
        
        get_path = kargs.get("get_path", False)

        if len(target_nodes) == 1 and type(target_nodes[0]) \
                in set([set, tuple, list, frozenset]):
            target_nodes = target_nodes[0]

        # Convert node names into node instances
        target_nodes = _translate_nodes(self, *target_nodes)

        # If only one node is provided, use self as the second target
        if type(target_nodes) != list:
            target_nodes = [target_nodes, self]
        elif len(target_nodes)==1:
            target_nodes = tree_nodes.append(self)

        n2path = {}
        reference = []
        ref_node = None
        for n in target_nodes:
            current = n
            while current: 
                n2path.setdefault(n, set()).add(current)
                if not ref_node:
                    reference.append(current)
                current = current.up
            if not ref_node:
                ref_node = n

        common = None
        for n in reference:
            broken = False
            for node, path in n2path.items():
                if node is not ref_node and n not in path:
                    broken = True
                    break

            if not broken: 
                common = n
                break
        if not common: 
            raise ValueError("Nodes are not connected!")

        if get_path:
            return common, n2path
        else:
            return common

    def iter_search_nodes(self, **conditions):
        """ 
        Search nodes in an interative way. Matches are being yield as
        they are being found. This avoids to scan the full tree
        topology before returning the first matches. Useful when
        dealing with huge trees.
        """
        
        for n in self.traverse():
            conditions_passed = 0
            for key, value in conditions.items():
                if hasattr(n, key) and getattr(n, key) == value:
                    conditions_passed +=1
            if conditions_passed == len(conditions):
                yield n

    def search_nodes(self, **conditions):
        """
        Returns the list of nodes matching a given set of conditions.
        **Example:**
        ::
          tree.search_nodes(dist=0.0, name="human")
        """
        matching_nodes = []
        for n in self.iter_search_nodes(**conditions):
            matching_nodes.append(n)
        return matching_nodes

    def get_leaves_by_name(self,name):
        """ 
        Returns a list of leaf nodes matching a given name. 
        """
        return self.search_nodes(name=name, children=[])

    def is_leaf(self):
        """ 
        Return True if current node is a leaf.
        """
        return len(self.children) == 0

    def is_root(self):
        """ 
        Returns True if current node has no parent
        """
        if self.up is None:
            return True
        else:
            return False

    # ###########################
    # Distance related functions
    # ###########################
    def get_distance(self, target, target2=None, topology_only=False):
        """
        Returns the distance between two nodes. If only one target is
        specified, it returns the distance bewtween the target and the
        current node.
        
        :argument target: a node within the same tree structure.
        :argument target2: a node within the same tree structure. If
          not specified, current node is used as target2.
        :argument False topology_only: If set to True, distance will
          refer to the number of nodes between target and target2.
        :returns: branch length distance between target and
          target2. If topology_only flag is True, returns the number
          of nodes between target and target2.
 
        """

        if target2 is None:
            target2 = self
            root = self.get_tree_root()
        else:
            # is target node under current node?
            root = self

        target, target2 = _translate_nodes(root, target, target2)
        ancestor = root.get_common_ancestor(target, target2)
        if ancestor is None:
            raise TreeError("Nodes are not connected")

        dist = 0.0
        for n in [target2, target]:
            current = n
            while current != ancestor:
                if topology_only:
                    if  current!=target:
                        dist += 1
                else:
                    dist += current.dist
                current = current.up
        return dist

    def get_farthest_node(self, topology_only=False):
        """
        Returns the node's farthest descendant or ancestor node, and the
        distance to it.
        :argument False topology_only: If set to True, distance
          between nodes will be referred to the number of nodes
          between them. In other words, topological distance will be
          used instead of branch length distances.
        :return: A tuple containing the farthest node referred to the
          current node and the distance to it.
        """
        # Init fasthest node to current farthest leaf
        farthest_node,farthest_dist = self.get_farthest_leaf(topology_only=topology_only)
        prev    = self
        if topology_only:
            cdist = 0
        else:
            cdist = prev.dist
        current = prev.up
        while current is not None:
            for ch in current.children:
                if ch != prev:
                    if not ch.is_leaf():
                        fnode, fdist = ch.get_farthest_leaf(topology_only=topology_only)
                    else:
                        fnode = ch
                        fdist = 0
                    if topology_only:
                        fdist += 1.0
                    else:
                        fdist += ch.dist
                    if cdist+fdist > farthest_dist:
                        farthest_dist = cdist + fdist
                        farthest_node = fnode
            prev = current
            if topology_only:
                cdist += 1
            else:
                cdist  += prev.dist
            current = prev.up
        return farthest_node, farthest_dist

    def get_farthest_leaf(self, topology_only=False):
        """
        Returns node's farthest descendant node (which is always a leaf), and the
        distance to it.
        :argument False topology_only: If set to True, distance
          between nodes will be referred to the number of nodes
          between them. In other words, topological distance will be
          used instead of branch length distances.
        :return: A tuple containing the farthest leaf referred to the
          current node and the distance to it.
        """
        max_dist = 0.0
        max_node = None
        if self.is_leaf():
            return self, 0.0
        else:
            for ch in self.children:
                node, d = ch.get_farthest_leaf(topology_only=topology_only)
                if topology_only:
                    d += 1.0
                else:
                    d += ch.dist
                if d>=max_dist:
                    max_dist = d
                    max_node = node
            return max_node, max_dist

    def get_closest_leaf(self, topology_only=False):
        """Returns node's closest descendant leaf and the distance to
        it.
        :argument False topology_only: If set to True, distance
          between nodes will be referred to the number of nodes
          between them. In other words, topological distance will be
          used instead of branch length distances.
        :return: A tuple containing the closest leaf referred to the
          current node and the distance to it.
        """
        min_dist = None
        min_node = None
        if self.is_leaf():
            return self, 0.0
        else:
            for ch in self.children:
                node, d = ch.get_closest_leaf(topology_only=topology_only)
                if topology_only:
                    d += 1.0
                else:
                    d += ch.dist
                if min_dist is None or d<min_dist:
                    min_dist = d
                    min_node = node
            return min_node, min_dist


            
    def get_midpoint_outgroup(self):
        """
        Returns the node that divides the current tree into two distance-balanced
        partitions.
        """
        # Gets the farthest node to the current root
        root = self.get_tree_root()
        nA , r2A_dist = root.get_farthest_leaf()
        nB , A2B_dist = nA.get_farthest_node()

        outgroup = nA
        middist  = A2B_dist / 2.0
        cdist = 0
        current = nA
        while current is not None:
            cdist += current.dist
            if cdist > (middist): # Deja de subir cuando se pasa del maximo
                break
            else:
                current = current.up
        return current
        
    def populate(self, size, names_library=None, reuse_names=False,
                 random_branches=False, branch_range=(0,1),
                 support_range=(0,1)): 
        """
        Generates a random topology by populating current node.
        :argument None names_library: If provided, names library
          (list, set, dict, etc.) will be used to name nodes.
        :argument False reuse_names: If True, node names will not be
          necessarily unique, which makes the process a bit more
          efficient.
        :argument False random_branches: If True, branch distances and support
          values will be randomized.
        
        :argument (0,1) branch_range: If random_branches is True, this
        range of values will be used to generate random distances.
        :argument (0,1) support_range: If random_branches is True,
        this range of values will be used to generate random branch
        support values.
        """
        NewNode = self.__class__

        if len(self.children) > 1: 
            connector = NewNode()
            for ch in self.get_children():
                ch.detach()
                connector.add_child(child = ch)
            root = NewNode()
            self.add_child(child = connector)
            self.add_child(child = root)
        else:
            root = self

        next = deque([root])
        for i in range(size-1):
            if random.randint(0, 1):
                p = next.pop()
            else:
                p = next.popleft()

            c1 = p.add_child()
            c2 = p.add_child()
            next.extend([c1, c2])
            if random_branches:
                c1.dist = random.uniform(*branch_range)
                c2.dist = random.uniform(*branch_range)
                c1.support = random.uniform(*branch_range)
                c2.support = random.uniform(*branch_range)
            else:
                c1.dist = 1.0
                c2.dist = 1.0
                c1.support = 1.0
                c2.support = 1.0

        # next contains leaf nodes
        charset =  "abcdefghijklmnopqrstuvwxyz"
        if names_library:
            names_library = deque(names_library)
        else:
            avail_names = itertools.combinations_with_replacement(charset, 10)
        for n in next:
            if names_library:
                if reuse_names: 
                    tname = random.sample(names_library, 1)[0]
                else:
                    tname = names_library.pop()
            else:
                tname = ''.join(next(avail_names))
            n.name = tname
            

    def set_outgroup(self, outgroup):
        """
        Sets a descendant node as the outgroup of a tree.  This function
        can be used to root a tree or even an internal node.
        :argument outgroup: a node instance within the same tree
          structure that will be used as a basal node. 
        """

        outgroup = _translate_nodes(self, outgroup)

        if self == outgroup:
            raise ValueError("Cannot set myself as outgroup")

        parent_outgroup = outgroup.up

        # Detects (sub)tree root
        n = outgroup
        while n.up is not self:
            n = n.up

        # If outgroup is a child from root, but with more than one
        # sister nodes, creates a new node to group them

        self.children.remove(n)
        if len(self.children) != 1:
            down_branch_connector = self.__class__()
            down_branch_connector.dist = 0.0
            down_branch_connector.support = n.support
            for ch in self.get_children():
                down_branch_connector.children.append(ch)
                ch.up = down_branch_connector
                self.children.remove(ch)
        else:
            down_branch_connector = self.children[0]

        # Connects down branch to myself or to outgroup
        quien_va_ser_padre = parent_outgroup
        if quien_va_ser_padre is not self:
            # Parent-child swapping
            quien_va_ser_hijo = quien_va_ser_padre.up
            quien_fue_padre = None
            buffered_dist = quien_va_ser_padre.dist
            buffered_support = quien_va_ser_padre.support
            
            while quien_va_ser_hijo is not self:
                quien_va_ser_padre.children.append(quien_va_ser_hijo)
                quien_va_ser_hijo.children.remove(quien_va_ser_padre)

                buffered_dist2 = quien_va_ser_hijo.dist
                buffered_support2 = quien_va_ser_hijo.support
                quien_va_ser_hijo.dist = buffered_dist
                quien_va_ser_hijo.support = buffered_support
                buffered_dist = buffered_dist2
                buffered_support = buffered_support2

                quien_va_ser_padre.up = quien_fue_padre
                quien_fue_padre = quien_va_ser_padre

                quien_va_ser_padre = quien_va_ser_hijo
                quien_va_ser_hijo = quien_va_ser_padre.up

            quien_va_ser_padre.children.append(down_branch_connector)
            down_branch_connector.up = quien_va_ser_padre
            quien_va_ser_padre.up = quien_fue_padre

            down_branch_connector.dist += buffered_dist
            outgroup2 = parent_outgroup
            parent_outgroup.children.remove(outgroup)
            outgroup2.dist = 0
           
        else:
            outgroup2 = down_branch_connector

        outgroup.up = self
        outgroup2.up = self
        # outgroup is always the first children. Some function my
        # trust on this fact, so do no change this.
        self.children = [outgroup,outgroup2]
        middist = (outgroup2.dist + outgroup.dist)/2
        outgroup.dist = middist
        outgroup2.dist = middist
        outgroup2.support = outgroup.support

    def unroot(self):
        """ 
        Unroots current node. This function is expected to be used on
        the absolute tree root node, but it can be also be applied to
        any other internal node. It will convert a split into a
        multifurcation.
        """
        # if is rooted
        if not self.is_root():
            sys.stderr.write("Warning. You are unrooting an internal node!!!\n")
        if len(self.children)==2:
            if not self.children[0].is_leaf():
                self.children[0].delete()
            elif not self.children[1].is_leaf():
                self.children[1].delete()
            else:
                raise TreeError("Cannot unroot a tree with only two leaves")

    def copy(self, method="cpickle"):
        """.. versionadded: 2.1
        Returns a copy of the current node.
        :var cpickle method: Protocol used to copy the node
        structure. The following values are accepted:
           - "newick": Tree topology, node names, branch lengths and
             branch support values will be copied by as represented in
             the newick string (copy by newick string serialisation).
        
           - "newick-extended": Tree topology and all node features
             will be copied based on the extended newick format
             representation. Only node features will be copied, thus
             excluding other node attributes. As this method is also
             based on newick serialisation, features will be converted
             into text strings when making the copy. 
           - "cpickle": The whole node structure and its content is
             cloned based on cPickle object serialisation (slower, but
             recommended for full tree copying)
        
           - "deepcopy": The whole node structure and its content is
             copied based on the standard "copy" Python functionality
             (this is the slowest method but it allows to copy complex
             objects even if attributes point to lambda functions,
             etc.)
        """
        if method=="newick":
            new_node = self.__class__(self.write(features=["name"], format_root_node=True))
        elif method=="newick-extended":
            self.write(features=[], format_root_node=True)
            new_node = self.__class__(self.write(features=[]))
        elif method == "deepcopy":
            parent = self.up
            self.up = None
            new_node = copy.deepcopy(self)
            self.up = parent
        elif method == "cpickle":
            parent = self.up
            self.up = None
            new_node = pickle.loads(pickle.dumps(self, 2))
            self.up = parent
        else:
            raise ValuerError("Invalid copy method")
            
        return new_node
        
    def _asciiArt(self, char1='-', show_internal=True, compact=False, attributes=None):
        """
        Returns the ASCII representation of the tree.
        Code based on the PyCogent GPL project.
        """
        if not attributes:
            attributes = ["name"]
        node_name = ', '.join(map(str, [getattr(self, v) for v in attributes if hasattr(self, v)]))
        
        LEN = max(3, len(node_name) if not self.children or show_internal else 3)
        PAD = ' ' * LEN
        PA = ' ' * (LEN-1)
        if not self.is_leaf():
            mids = []
            result = []
            for c in self.children:
                if len(self.children) == 1:
                    char2 = '/'
                elif c is self.children[0]:
                    char2 = '/'
                elif c is self.children[-1]:
                    char2 = '\\'
                else:
                    char2 = '-'
                (clines, mid) = c._asciiArt(char2, show_internal, compact, attributes)
                mids.append(mid+len(result))
                result.extend(clines)
                if not compact:
                    result.append('')
            if not compact:
                result.pop()
            (lo, hi, end) = (mids[0], mids[-1], len(result))
            prefixes = [PAD] * (lo+1) + [PA+'|'] * (hi-lo-1) + [PAD] * (end-hi)
            mid = int((lo + hi) / 2)
            prefixes[mid] = char1 + '-'*(LEN-2) + prefixes[mid][-1]
            result = [p+l for (p,l) in zip(prefixes, result)]
            if show_internal:
                stem = result[mid]
                result[mid] = stem[0] + node_name + stem[len(node_name)+1:]
            return (result, mid)
        else:
            return ([char1 + '-' + node_name], 0)

    def get_ascii(self, show_internal=True, compact=False, attributes=None):
        """
        Returns a string containing an ascii drawing of the tree.
        :argument show_internal: includes internal edge names.
        :argument compact: use exactly one line per tip.
        :param attributes: A list of node attributes to shown in the
            ASCII representation.
        
        """
        (lines, mid) = self._asciiArt(show_internal=show_internal,
                                      compact=compact, attributes=attributes)
        return '\n'+'\n'.join(lines)


    def ladderize(self, direction=0):
        """ 
        .. versionadded: 2.1 
        Sort the branches of a given tree (swapping children nodes)
        according to the size of each partition.
        ::
           t =  Tree("(f,((d, ((a,b),c)),e));")
           print t
           #            
           #      /-f
           #     |
           #     |          /-d
           # ----|         |
           #     |     /---|          /-a
           #     |    |    |     /---|
           #     |    |     \---|     \-b
           #      \---|         |
           #          |          \-c
           #          |
           #           \-e
           t.ladderize()
           print t
           #      /-f
           # ----|
           #     |     /-e
           #      \---|
           #          |     /-d
           #           \---|
           #               |     /-c
           #                \---|
           #                    |     /-a
           #                     \---|
           #                          \-b
        """
       
        if not self.is_leaf():
            n2s = {}
            for n in self.get_children():
                s = n.ladderize(direction=direction)
                n2s[n] = s

            self.children.sort(lambda x,y: cmp(n2s[x], n2s[y]))
            if direction == 1:
                self.children.reverse()
            size = sum(n2s.values())
        else:
            size = 1

        return size

    def sort_descendants(self):
        """ 
        .. versionadded: 2.1 
        This function sort the branches of a given tree by
        considerening node names. After the tree is sorted, nodes are
        labeled using ascendent numbers.  This can be used to ensure
        that nodes in a tree with the same node names are always
        labeled in the same way. Note that if duplicated names are
        present, extra criteria should be added to sort nodes.
        Unique id is stored as a node._nid attribute
       
        """

        node2content = self.get_cached_content()
        def sort_by_content(x, y):
            return cmp(str(sorted([i.name for i in node2content[x]])),
                       str(sorted([i.name for i in node2content[y]])))

        for n in self.traverse():
            if not n.is_leaf():
                n.children.sort(sort_by_content)
        return node2content

    # def get_cached_content(self, store_attr=None,  _store=None):
    #     """ 
    #     .. versionadded: 2.2
       
    #     Returns a dictionary pointing to the preloaded content of each
    #     internal node under this tree. Such a dictionary is intended
    #     to work as a cache for operations that require many traversal
    #     operations.

    #     :param None store_attr: Specifies the node attribute that
    #     should be cached (i.e. name, distance, etc.). When none, the
    #     whole node instance is cached.

    #     :param _store: (internal use)

    #     """
    #     if _store is None:
    #         _store = {}
            
    #     for ch in self.children:
    #         ch.get_cached_content(store_attr=store_attr, _store=_store)

    #     if self.children:
    #         val = set()
    #         for ch in self.children:
    #             val.update(_store[ch])
    #         _store[self] = val
    #     else:
    #         if store_attr is None:
    #             val = self
    #         else:
    #             val = getattr(self, store_attr)
    #         _store[self] = set([val])
    #     return _store

    
    # def get_cached_content_non_recursive(self, store_attr=None,  _store=None):
    def get_cached_content(self, store_attr=None,  _store=None):
        """ 
        Returns a dictionary pointing to the preloaded content of each
        internal node under this tree. Such a dictionary is intended
        to work as a cache for operations that require many traversal
        operations.
        :param None store_attr: Specifies the node attribute that
        should be cached (i.e. name, distance, etc.). When none, the
        whole node instance is cached.
        :param _store: (internal use)
        """
        if _store is None:
            _store = {}

        for n in self.traverse("postorder"):
            if n.children:
                val = set()
                for ch in n.children:
                    val.update(_store[ch])
                _store[n] = val
            else:
                if store_attr is None:
                    val = n
                else:
                    val = getattr(n, store_attr)
                _store[n] = set([val])
        return _store

       
    def robinson_foulds(self, t2, attr_t1="name", attr_t2="name"):
        """
        .. versionadded: 2.2
        
        Returns the Robinson-Foulds symmetric distance between current
        tree and a different tree instance.
     
        :param t2: target tree
        
        :param name attr_t1: Compare trees using a custom node
                              attribute as a node name.
        
        :param name attr_t2: Compare trees using a custom node
                              attribute as a node name in target tree.
        :returns: (symmetric distance, total partitions, common node
         names, partitions in current tree, partitions in target tree)
           
        """
        
        t1 = self
        t1content = t1.get_cached_content()
        t2content = t2.get_cached_content()
        target_names = set([getattr(_n, attr_t1) for _n in t1content[t1]])
        ref_names = set([getattr(_n, attr_t2) for _n in t2content[t2]])
        common_names = target_names & ref_names
        if len(common_names) < 2:
            raise ValueError("Trees share less than 2 nodes")

        r1 = set([",".join(sorted([getattr(_c, attr_t1) for _c in cont
                                   if getattr(_c, attr_t1) in common_names]))
                  for cont in t1content.values() if len(cont)>1])
        r2 = set([",".join(sorted([getattr(_c, attr_t2) for _c in cont
                                   if getattr(_c, attr_t2) in common_names]))
                  for cont in t2content.values() if len(cont)>1])
        r1.discard("")
        r2.discard("")              
        inters = r1.intersection(r2)
        if len(r1) == len(r2):
                rf = (len(r1) - len(inters)) * 2
        else :
                rf = (len(r1) - len(inters)) + (len(r2) - len(inters))
        max_parts = len(r1) + len(r2)
        return rf, max_parts, common_names, r1, r2

    def get_partitions(self):
        """ 
        .. versionadded: 2.1
        
        It returns the set of all possible partitions under a
        node. Note that current implementation is quite inefficient
        when used in very large trees.
        t = Tree("((a, b), e);")
        partitions = t.get_partitions()
        # Will return: 
        # a,b,e
        # a,e
        # b,e
        # a,b
        # e
        # b
        # a
        """
        all_leaves = frozenset(self.get_leaf_names())
        all_partitions = set([all_leaves])
        for n in self.iter_descendants():
            p1 = frozenset(n.get_leaf_names())
            p2 = frozenset(all_leaves - p1)
            all_partitions.add(p1)
            all_partitions.add(p2)
        return all_partitions

    def convert_to_ultrametric(self, tree_length, strategy="balanced"):
        """
        .. versionadded: 2.1 
        Converts a tree to ultrametric topology (all leaves must have
        the same distance to root). Note that, for visual inspection
        of ultrametric trees, node.img_style["size"] should be set to
        0.
        """

        # pre-calculate how many splits remain under each node
        node2max_depth = {}
        for node in self.traverse("postorder"):
            if not node.is_leaf():
                max_depth = max([node2max_depth[c] for c in node.children]) + 1
                node2max_depth[node] = max_depth
            else:
                node2max_depth[node] = 1
        node2dist = {self: 0.0}
        tree_length = float(tree_length)
        step = tree_length / node2max_depth[self]
        for node in self.iter_descendants("levelorder"):
            if strategy == "balanced":
                node.dist = (tree_length - node2dist[node.up]) / node2max_depth[node]
                node2dist[node] =  node.dist + node2dist[node.up]
            elif strategy == "fixed":
                if not node.is_leaf():
                    node.dist = step
                else:
                    node.dist = tree_length - ((node2dist[node.up]) * step)
                node2dist[node] = node2dist[node.up] + 1
            node.dist = node.dist

    def check_monophyly(self, values, target_attr, ignore_missing=False):
        """
        Returns True if a given target attribute is monophyletic under
        this node for the provided set of values. 
        If not all values are represented in the current tree
        structure, a ValueError exception will be raised to warn that
        strict monophyly could never be reached (this behaviour can be
        avoided by enabling the `ignore_missing` flag.
        :param values: a set of values for which monophyly is
            expected.
            
        :param target_attr: node attribute being used to check
            monophyly (i.e. species for species trees, names for gene
            family trees, or any custom feature present in the tree).
        :param False ignore_missing: Avoid raising an Exception when
            missing attributes are found. 
        """
        if type(values) != set:
            values = set(values)

        # This is the only time I traverse the tree, then I use cached
        # leaf content
        n2leaves = self.get_cached_content()
        # Locate leaves matching requested attribute values
        targets = [leaf for leaf in n2leaves[self]
                   if getattr(leaf, target_attr) in values]
       
        # Raise an error if requested attribute values are not even present
        if not ignore_missing:
            missing_values = values - set([getattr(n, target_attr) for n
                                           in targets])
            if missing_values: 
                raise ValueError("Expected '%s' value(s) not found: %s" %(
                    target_attr, ','.join(missing_values)))
                
        # Check monophyly with get_common_ancestor. Note that this
        # step does not require traversing the tree again because
        # targets are node instances instead of node names, and
        # get_common_ancestor function is smart enough to detect it
        # and avoid unnecessary traversing.
        common = self.get_common_ancestor(targets)
        observed = n2leaves[common]
        foreign_leaves = [leaf for leaf in observed
                          if getattr(leaf, target_attr) not in values]
        if not foreign_leaves:
            return True, "monophyletic"
        else:
            # if the requested attribute is not monophyletic in this
            # node, let's differentiate between poly and paraphyly. 
            poly_common = self.get_common_ancestor(foreign_leaves)
            # if the common ancestor of all foreign leaves is self
            # contained, we have a paraphyly. Otherwise, polyphyly. 
            polyphyletic = [leaf for leaf in poly_common if
                            getattr(leaf, target_attr) in values]
            if polyphyletic:
                return False, "polyphyletic"
            else:
                return False, "paraphyletic"

            
    def get_monophyletic(self, values, target_attr):
        """
        .. versionadded:: 2.2
        Returns a list of nodes matching the provided monophyly
        criteria. For a node to be considered a match, all
        `target_attr` values within and node, and exclusively them,
        should be grouped.
        
        :param values: a set of values for which monophyly is
            expected.
            
        :param target_attr: node attribute being used to check
            monophyly (i.e. species for species trees, names for gene
            family trees).
           
        """

        if type(values) != set:
            values = set(values)

        n2values = self.get_cached_content(store_attr=target_attr)
      
        is_monophyletic = lambda node: n2values[node] == values
        for match in self.iter_leaves(is_leaf_fn=is_monophyletic):
            if is_monophyletic(match):
                yield match

            
    def resolve_polytomy(self, default_dist=0.0, default_support=0.0,
                         recursive=True):
        """
        .. versionadded: 2.2
        
        Resolve all polytomies under current node by creating an
        arbitrary dicotomic structure among the affected nodes. This
        function randomly modifies current tree topology and should
        only be used for compatibility reasons (i.e. programs
        rejecting multifurcated node in the newick representation).
        :param 0.0 default_dist: artificial branch distance of new
            nodes.
                                  
        :param 0.0 default_support: artificial branch support of new
            nodes.
                                   
        :param True recursive: Resolve any polytomy under this
             node. When False, only current node will be checked and fixed.
        """
        
        
        def _resolve(node):
            if len(node.children) > 2:
                children = list(node.children)
                node.children = []
                next_node = root = node
                for i in range(len(children)-2):
                    next_node = next_node.add_child()
                    next_node.dist = default_dist
                    next_node.support = default_support

                next_node = root
                for ch in children:
                    next_node.add_child(ch)
                    if ch != children[-2]:
                        next_node = next_node.children[0]
        target = [self]
        if recursive: 
            target.extend([n for n in self.get_descendants()])
        for n in target:
            _resolve(n)


def _translate_nodes(root, *nodes):
    name2node = dict([ [n, None] for n in nodes if type(n) is str])
    for n in root.traverse():
        if n.name in name2node:
            if name2node[n.name] is not None:
                raise ValueError("Ambiguous node name: "+str(n.name))
            else:
                name2node[n.name] = n

    if None in name2node.values():
        notfound = [key for key, value in name2node.items() if value is None]
        raise ValueError("Node names not found: "+str(notfound))

    valid_nodes = []
    for n in nodes: 
        if type(n) is not str:
            if type(n) is not root.__class__ :
                raise ValueError("Invalid target node: "+str(n))
            else:
                valid_nodes.append(n)
            
    valid_nodes.extend(name2node.values())
    if len(valid_nodes) == 1:
        return valid_nodes[0]
    else:
        return valid_nodes


# Alias
#: .. currentmodule:: ete2
Tree = TreeNode






from lib.common.genome.genome_feature2 import Gene, GeneSet
from collections import OrderedDict
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.util import printer_list
import pandas as pd
# from lib.common.evolution.tree_operate import get_top_taxon_clade, get_root_by_species, add_clade_name, lookup_by_names, reroot_by_outgroup_clade, collapse_low_support, get_rooted_tree_by_species_tree
from lib.common.evolution.tree_operate import get_top_taxon_clade
from Bio import Phylo

__author__ = 'Yuxing Xu'


def if_conserved(species_list, conserved_arguments):
    """
    conserved_arguments = [
        [['Cau', 'Ini', 'Sly'], ['Ath', 'Aco']],
        [2,2]
    ]
    """
    sp_group_lol = conserved_arguments[0]
    group_min_num_list = conserved_arguments[1]

    conserved_flag = True

    for sp_group_list, min_num in zip(sp_group_lol, group_min_num_list):
        if len(set(species_list) & set(sp_group_list)) < min_num:
            conserved_flag = False

    return conserved_flag


class OrthoGroup(GeneSet):
    def __init__(self, id=None, from_gene_list=None, species_list=None, from_OG_dict=None, properties=None):
        if from_OG_dict and from_gene_list is None:
            if not species_list:
                species_list = list(from_OG_dict.keys())

            from_gene_list = []
            for sp_id in from_OG_dict:
                for g_id in from_OG_dict[sp_id]:
                    from_gene_list.append(Gene(id=g_id, species=sp_id))

        if species_list:
            gene_dict = {i: [] for i in species_list}
        else:
            gene_dict = {}
            for i in from_gene_list:
                gene_dict[i.species] = []

        gene_list = [i for i in from_gene_list if i.species in gene_dict]

        super(OrthoGroup, self).__init__(id=id, gene_list=gene_list)

        self.gene_dict = gene_dict
        for i in self.gene_list:
            self.gene_dict[i.species].append(i)

        self.species_stat = {}
        for sp_id in self.gene_dict:
            self.species_stat[sp_id] = len(self.gene_dict[sp_id])

        self.species_list = list(self.species_stat.keys())

        self.properties = properties

    def conserved(self, conserved_arguments):
        non_zero_species_list = [
            i for i in self.species_stat if self.species_stat[i] != 0]

        return if_conserved(non_zero_species_list, conserved_arguments)


class OrthoGroups(object):
    def __init__(self, id=None, OG_tsv_file=None, from_OG_dict=None, from_OG_list=None, species_list=None, note=None, new_OG_name=False):
        self.id = id
        self.note = note
        self.OG_tsv_file = OG_tsv_file
        self.from_OG_dict = from_OG_dict

        if OG_tsv_file:
            OG_dict = self.build_from_OG_tsv_file(
                OG_tsv_file, species_list=species_list)
        elif from_OG_dict:
            OG_dict = self.build_from_OG_dict(
                from_OG_dict, species_list=species_list)
        elif from_OG_list:
            OG_dict = self.build_from_OG_list(
                from_OG_list, species_list=species_list, new_OG_name=new_OG_name)
        else:
            OG_dict = OrderedDict()

        self.OG_dict = OG_dict
        if species_list:
            self.species_list = species_list
        else:
            self.species_list = OG_dict[list(OG_dict.keys())[0]].species_list
        self.OG_id_list = list(self.OG_dict.keys())

    def get(self, key_tuple):
        """
        key_tuple: ('OG1234', 'Ath'), or ('OG1234', None), or (None, 'Ath')
        """

        OG_id, sp_id = key_tuple

        if OG_id and sp_id:
            return self.OG_dict[OG_id].gene_dict[sp_id]
        elif OG_id:
            return self.OG_dict[OG_id].gene_dict
        elif sp_id:
            output_dict = {}
            for i in self.OG_dict:
                output_dict[i] = self.OG_dict[i].gene_dict[sp_id]
            return output_dict

    def build_from_OG_tsv_file(self, OG_tsv_file, species_list=None):
        file_info = tsv_file_dict_parse(OG_tsv_file)

        full_species_list = list(
            file_info[list(file_info.keys())[0]].keys())[1:]

        OG_dict = OrderedDict()

        for i in file_info:
            OG_id = file_info[i]['Orthogroup']

            gene_list = []
            for j in file_info[i]:
                if j == 'Orthogroup':
                    continue
                if file_info[i][j] == '' or file_info[i][j] is None:
                    continue
                gene_list.extend([Gene(id=k, species=j)
                                  for k in file_info[i][j].split(", ")])

            if species_list:
                OG_dict[OG_id] = OrthoGroup(
                    id=OG_id, from_gene_list=gene_list, species_list=species_list)
            else:
                OG_dict[OG_id] = OrthoGroup(
                    id=OG_id, from_gene_list=gene_list, species_list=full_species_list)

        return OG_dict

    def build_from_OG_dict(self, OG_dict, species_list=None):
        """
        OG_dict = {
            "OG0001": {
                "Ath": ['AT0001', 'AT0002'],
                "Osa": ['OS0001', 'OS0002']
            }
        }
        """
        new_OG_dict = OrderedDict()

        for OG_id in OG_dict:

            if isinstance(OG_dict[OG_id], dict):
                gene_list = []
                for sp_id in OG_dict[OG_id]:
                    gene_list.extend([Gene(id=i, species=sp_id)
                                      for i in OG_dict[OG_id][sp_id]])
                new_OG_dict[OG_id] = OrthoGroup(
                    id=OG_id, from_gene_list=gene_list, species_list=species_list)
            else:
                new_OG_dict[OG_id] = OG_dict[OG_id]

        return new_OG_dict

    def build_from_OG_list(self, OG_list, species_list=None, new_OG_name=False):
        if species_list:
            all_species_list = species_list
        else:
            all_species_list = []
            for og in OG_list:
                all_species_list.extend(og.species_list)
            all_species_list = list(set(all_species_list))

        new_OG_dict = OrderedDict()

        num = 0
        for og in OG_list:
            if new_OG_name:
                og_id = "OG%d" % num
                num += 1
            else:
                og_id = og.id

            gene_list = [
                gene for gene in og.gene_list if gene.species in set(all_species_list)]
            new_OG_dict[og_id] = OrthoGroup(
                id=og_id, from_gene_list=gene_list, species_list=all_species_list)

        return new_OG_dict

    def write_OG_tsv_file(self, output_file, species_list=None):
        if not species_list:
            species_list = self.species_list

        with open(output_file, 'w') as f:
            f.write(printer_list(species_list, head='Orthogroup\t')+"\n")

            for OG_id in self.OG_dict:
                OG = self.OG_dict[OG_id]
                string_list = []
                for sp_id in species_list:
                    string_list.append(printer_list(
                        [i.id for i in OG.gene_dict[sp_id]], sep=', '))
                output_string = printer_list(string_list, head=OG_id+"\t")
                f.write(output_string+"\n")

        return output_file

    def write_OG_num_file(self, output_file, species_list=None):
        if not species_list:
            species_list = self.species_list

        with open(output_file, 'w') as f:
            f.write(printer_list(species_list, head='Orthogroup\t')+"\n")

            for OG_id in self.OG_dict:
                OG = self.OG_dict[OG_id]
                string_list = []
                for sp_id in species_list:
                    string_list.append(len(
                        [i.id for i in OG.gene_dict[sp_id]]))
                output_string = printer_list(string_list, head=OG_id+"\t")
                f.write(output_string+"\n")

        return output_file

    def get_conserved_OG_list(self, conserved_arguments):

        output_list = []
        for i in self.OG_dict:  
            og = self.OG_dict[i]
            if og.conserved(conserved_arguments):
                output_list.append(i)

        return output_list

    def remove_some_OG(self, removed_OG_list):
        for i in removed_OG_list:
            self.OG_id_list.remove(i)
            del self.OG_dict[i]


class Species(object):
    def __init__(self,
                 sp_id=None,
                 taxon_id=None,
                 species_name=None,
                 genome_file=None,
                 gff_file=None,
                 pt_file=None,
                 cDNA_file=None,
                 cds_file=None):

        self.sp_id = sp_id
        self.taxon_id = taxon_id
        self.species_name = species_name
        self.genome_file = genome_file
        self.gff_file = gff_file
        self.pt_file = pt_file
        self.cDNA_file = cDNA_file
        self.cds_file = cds_file


def read_species_info(species_info_table, add_option=[]):
    species_info_df = pd.read_excel(species_info_table)
    species_info_dict = {}
    for index in species_info_df.index:
        sp_id = str(species_info_df.loc[index]['sp_id'])

        base_info_dict = {i: None for i in [
            'taxon_id', 'species_name', 'genome_file', 'gff_file', 'pt_file', 'cDNA_file', 'cds_file']}

        for i in base_info_dict:
            if i in species_info_df.loc[index]:
                base_info_dict[i] = species_info_df.loc[index][i]

        species_info_dict[sp_id] = Species(
            sp_id, base_info_dict['taxon_id'], base_info_dict['species_name'], base_info_dict['genome_file'], base_info_dict['gff_file'], base_info_dict['pt_file'], base_info_dict['cDNA_file'], base_info_dict['cds_file'])

        for i in add_option:
            if i in species_info_df.loc[index]:
                setattr(species_info_dict[sp_id], i,
                        species_info_df.loc[index][i])

    return species_info_dict


# about get orthoclade from tree
def treeclade_to_OG(clade, species_list, og_id=None):
    if not og_id:
        og_id = clade.name

    # g_id_list = [leaf.name for leaf in clade.get_terminals()]
    gene_list = [Gene(id=leaf.name, species=leaf.taxon)
                 for leaf in clade.get_terminals()]
    og = OrthoGroup(id=og_id, from_gene_list=gene_list,
                    species_list=species_list)

    return og


def get_orthogroups_from_gene_tree(taxon_labeled_tree, species_tree, taxonomy_level=None, conserved_arguments=None):
    all_sp_list = [i.name for i in species_tree.get_terminals()]

    if taxonomy_level is None:
        taxonomy_level = species_tree.root.name

    if conserved_arguments is None:
        conserved_arguments = [
            [all_sp_list],
            [0]
        ]

    top_speciation_clade_list = get_top_taxon_clade(
        taxon_labeled_tree, species_tree, taxonomy_level, node_type='speciation')

    og_list = []
    for clade in top_speciation_clade_list:
        clade_taxon_list = [i.taxon for i in clade.get_terminals()]
        if if_conserved(clade_taxon_list, conserved_arguments):
            og = treeclade_to_OG(clade, all_sp_list)
            og_list.append(og)

    return og_list


# def get_conserved_clades(rooted_tree_node_dict, gene_to_species_map_dict, conserved_function, conserved_arguments):
#     """
#     filter all conserved clades on a rooted tree, return clades may overlap with each other
#     """
#     conserved_clades = []
#     for clade_id in rooted_tree_node_dict:
#         clade = rooted_tree_node_dict[clade_id]
#         if not clade.is_terminal():
#             sp_list = list(set([gene_to_species_map_dict[leaf.name]
#                                 for leaf in clade.get_terminals()]))
#             if conserved_function(sp_list, conserved_arguments):
#                 conserved_clades.append(clade_id)
#     return conserved_clades


# def get_orthogroups_from_tree(tree_prefix, gene_tree, species_tree, gene_to_species_map_dict, conserved_function, conserved_arguments, support_threshold):
#     """
#     warning: this function will get multi-level OGs at sametime
#     """
#     # root gene tree
#     gene_tree = add_clade_name(gene_tree)
#     gene_tree_node_dict = lookup_by_names(gene_tree)

#     best_root_clade = get_root_by_species(
#         gene_tree, species_tree, gene_to_species_map_dict)

#     gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(gene_tree,
#                                                                                                             gene_tree_node_dict,
#                                                                                                             best_root_clade.name,
#                                                                                                             True)

#     # collapse low support clades
#     gene_tree_rooted_collapsed = collapse_low_support(
#         gene_tree_rooted, support_threshold)
#     gene_tree_rooted_collapsed = add_clade_name(gene_tree_rooted_collapsed)
#     gene_tree_rooted_collapsed_node_dict = lookup_by_names(
#         gene_tree_rooted_collapsed)

#     # get conserved clades
#     conserved_clades_id_list = get_conserved_clades(
#         gene_tree_rooted_collapsed_node_dict, gene_to_species_map_dict, conserved_function, conserved_arguments)

#     # get sub_orthogroups
#     num = 0

#     orthogroup_id = "%s_%d" % (tree_prefix, num)
#     zero_clade_conserved = conserved_function(list(set(
#         [gene_to_species_map_dict[leaf.name] for leaf in gene_tree_rooted_collapsed.get_terminals()])), conserved_arguments)
#     sub_orthogroup_dict = {orthogroup_id: (
#         [leaf.name for leaf in gene_tree_rooted_collapsed.get_terminals()], zero_clade_conserved)}

#     for clade_id in conserved_clades_id_list:
#         num += 1
#         orthogroup_id = "%s_%d" % (tree_prefix, num)
#         clade = gene_tree_rooted_collapsed_node_dict[clade_id]
#         sub_orthogroup_dict[orthogroup_id] = (
#             [leaf.name for leaf in clade.get_terminals()], True)

#     for orthogroup_id in sub_orthogroup_dict:
#         tmp_dict = {}
#         for i in sub_orthogroup_dict[orthogroup_id][0]:
#             if not gene_to_species_map_dict[i] in tmp_dict:
#                 tmp_dict[gene_to_species_map_dict[i]] = []
#             tmp_dict[gene_to_species_map_dict[i]].append(i)
#         sub_orthogroup_dict[orthogroup_id] = tmp_dict

#     sp_list = [leaf.name for leaf in species_tree.get_terminals()]
#     sub_OGs = OrthoGroups(
#         id=tree_prefix, from_OG_dict=sub_orthogroup_dict, species_list=sp_list)

#     return sub_OGs


def if_sub_OG(A_OG, B_OG):
    """
    if A is a sub OG of B
    """

    A_gene_set = set([(i.id, i.species) for i in A_OG.gene_list])
    B_gene_set = set([(i.id, i.species) for i in B_OG.gene_list])

    return A_gene_set & B_gene_set == A_gene_set


def get_non_overlap_orthogroups(OGs):
    exclude_list = []
    for i in OGs.OG_id_list:
        for j in OGs.OG_id_list:
            if i == j:
                continue
            else:
                if if_sub_OG(OGs.OG_dict[i], OGs.OG_dict[j]):
                    exclude_list.append(j)

    exclude_list = set(exclude_list)

    used_OG_dict = OrderedDict()
    for i in OGs.OG_dict:
        if not i in exclude_list:
            used_OG_dict[i] = OGs.OG_dict[i]

    return OrthoGroups(from_OG_dict=used_OG_dict, species_list=OGs.species_list)


def merge_OGs(OGs_list):

    OG_dict = {}
    species_list = []
    for ogs in OGs_list:
        species_list.extend(ogs.species_list)
        for i in ogs.OG_dict:
            OG_dict[i] = ogs.OG_dict[i]
    species_list = list(set(species_list))

    return OrthoGroups(from_OG_dict=OG_dict, species_list=species_list)


def OG_gene_rename(OG, rename_dict):
    """
    rename_dict: key: old_id, value: new_id
    """

    gene_list_new = [Gene(id=rename_dict[gene.id], species=gene.species)
                     for gene in OG.gene_list]

    return OrthoGroup(id=OG.id, from_gene_list=gene_list_new, species_list=OG.species_list)


def OGs_gene_rename(OGs, rename_dict):
    """
    rename_dict: key: old_id, value: new_id
    """

    OG_dict = {}
    for i in OGs.OG_dict:
        og = OGs.OG_dict[i]
        new_og = OG_gene_rename(og, rename_dict)
        OG_dict[i] = new_og

    return OrthoGroups(id=OGs.id, OG_tsv_file=None, from_OG_dict=OG_dict, species_list=OGs.species_list, note=OGs.note)


if __name__ == '__main__':
    # read a OG tsv file
    OGs = OrthoGroups(
        OG_tsv_file="/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/pt_file/OrthoFinder/Results_May08/Orthogroups/Orthogroups.tsv")
    OGs.get(('OG0000000', 'Ath'))

    # read a standard species info file: sp_id, taxon_id, species_name, genome_file, gff_file, pt_file, cDNA_file, cds_file
    ref_xlsx = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/Gel_ref.xlsx'
    sp_info_dict = read_species_info(ref_xlsx)

    # get orthogroups from a gene tree (and species tree)
    from lib.common.fileIO import tsv_file_dict_parse
    from lib.common.evolution.tree_operate import map_node_species_info, add_clade_name, get_rooted_tree_by_species_tree

    normal_gene_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/phylogenomics_old/tree/fasttree/OG0002678/tree.resolve.phb"
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/species.tree.txt"
    rename_map = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/phylogenomics_old/tree/fasttree/OG0002678/rename.map"

    gene_tree = Phylo.read(normal_gene_tree_file, 'newick')
    gene_tree = add_clade_name(gene_tree)
    species_tree = Phylo.read(species_tree_file, 'newick')
    species_tree = add_clade_name(species_tree)

    tmp_info = tsv_file_dict_parse(rename_map, fieldnames=[
                                   'new_id', 'old_id', 'speci'], key_col='new_id')
    gene_to_species_map_dict = {i: tmp_info[i]["speci"] for i in tmp_info}

    rooted_gene_tree = get_rooted_tree_by_species_tree(
        gene_tree, species_tree, gene_to_species_map_dict)
    rooted_taxon_gene_tree = map_node_species_info(
        rooted_gene_tree, species_tree, gene_to_species_map_dict)

    taxonomy_level = 'Orchidaceae'
    conserved_arguments = [
        [['Gel', 'Peq', 'Dca', 'Vpl', 'Ash']],
        [3]
    ]

    og_list = get_orthogroups_from_gene_tree(
        rooted_taxon_gene_tree, species_tree, taxonomy_level=taxonomy_level, conserved_arguments=conserved_arguments)

    # get orthogroups from a nhx tree
    from lib.common.evolution.tree_operate import load_nhx_tree

    nhx_file = '/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/phylogenomics/tree/treebest/OG0001138/tree.nhx'
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/species.tree.txt"

    species_tree = Phylo.read(species_tree_file, 'newick')
    species_tree = add_clade_name(species_tree)

    rooted_taxon_gene_tree, clade_dict = load_nhx_tree(nhx_file)

    taxonomy_level = 'Orchidaceae'
    conserved_arguments = [
        [['Gel', 'Peq', 'Dca', 'Vpl', 'Ash']],
        [3]
    ]

    og_list = get_orthogroups_from_gene_tree(
        rooted_taxon_gene_tree, species_tree, taxonomy_level=taxonomy_level, conserved_arguments=conserved_arguments)



import pandas as pd
import numpy as np


def dod2df(dod, T=False):
    """
    conver a dict of dicts to a dataframe, sub_dict should have same keys and one dict will be a column

    input:
    dod = {
        'name' : {1:'xyx',2:'fkn'},
        'age' : {1:29,2:27}
    }

    output:
        name    age
    0   xyx     29
    1   fkn     27
    """

    dod_for_pd = {}
    for i in dod:
        dod_for_pd[i] = pd.Series(dod[i])

    return pd.DataFrame(dod_for_pd)


if __name__ == '__main__':
    pass



from lib.common.genome.genome_feature2 import read_bed_file, get_chrloci_overlap

input1_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/temp/Ath.rep.bed'
input2_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/temp/PC_Ath.bed'
input3_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/temp/Ath.pseudo.bed'

gf_dict1 = read_bed_file(input1_file)
gf_dict2 = read_bed_file(input2_file)
gf_dict3 = read_bed_file(input3_file)
gf_dict3_long = {i:gf_dict3[i] for i in gf_dict3 if gf_dict3[i].end - gf_dict3[i].start + 1 > 200}

similarity_type = 'shorter_overlap_coverage'
threshold = 0.6

def feature_compare(gf_dict1, gf_dict2, threshold, similarity_type):
    num = 0
    overlap_gf = []
    for i in gf_dict1:
        num += 1
        print(num)
        gf1 = gf_dict1[i]
        for j in gf_dict2:
            gf2 = gf_dict2[j]
            overlap_ratio = get_chrloci_overlap(gf1, gf2, similarity_type=similarity_type)
            if overlap_ratio >= threshold:
                overlap_gf.append((i, j))

    overlap_gf_dict1 = [gf_dict1_id for gf_dict1_id, gf_dict2_id in overlap_gf]
    overlap_gf_dict2 = [gf_dict2_id for gf_dict1_id, gf_dict2_id in overlap_gf]

    only_in_set1 = list(set(gf_dict1.keys()) - set(overlap_gf_dict1))
    only_in_set2 = list(set(gf_dict2.keys()) - set(overlap_gf_dict2))

    return overlap_gf_dict1, overlap_gf_dict2, only_in_set1, only_in_set2

c1_1v2, c2_1v2, o1_1v2, o2_1v2 = feature_compare(gf_dict1, gf_dict2, threshold, similarity_type)
c1_1v3, c3_1v3, o1_1v3, o3_1v3 = feature_compare(gf_dict1, gf_dict3_long, threshold, similarity_type)
c2_2v3, c3_2v3, o2_2v3, o3_2v3 = feature_compare(gf_dict2, gf_dict3_long, threshold, similarity_type)



from lib.common.genome.genome_feature2 import read_bed_file, get_chrloci_overlap

input1_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/temp/Ath.rep.bed'
input2_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/temp/PC_Ath.bed'
input3_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/temp/Ath.pseudo.bed'

gf_dict1 = read_bed_file(input1_file)
gf_dict2 = read_bed_file(input2_file)
gf_dict3 = read_bed_file(input3_file)
gf_dict3_long = {i:gf_dict3[i] for i in gf_dict3 if gf_dict3[i].end - gf_dict3[i].start + 1 > 200}
    num = 0
    overlap_gf = []
    for i in gf_dict1:
        num += 1
        print(num)
        gf1 = gf_dict1[i]
        for j in gf_dict2:
            gf2 = gf_dict2[j]
            overlap_ratio = get_chrloci_overlap(gf1, gf2, similarity_type=similarity_type)
            if overlap_ratio > 0:
                overlap_gf.append(overlap_ratio)
similarity_type = "jaccord_score"
    num = 0
    overlap_gf = []
    for i in gf_dict1:
        num += 1
        print(num)
        gf1 = gf_dict1[i]
        for j in gf_dict3_long:
            gf2 = gf_dict3_long[j]
            overlap_ratio = get_chrloci_overlap(gf1, gf2, similarity_type=similarity_type)
            if overlap_ratio > 0:
                overlap_gf.append(overlap_ratio)
overlap_gf
overlap_gf
with open("1.txt", 'w') as f:
    for i in overlap_gf:
        f.write(str(i)+"\n")
pwd
    num = 0
    overlap_gf = []
    for i in gf_dict2:
        num += 1
        print(num)
        gf1 = gf_dict2[i]
        for j in gf_dict3_long:
            gf2 = gf_dict3_long[j]
            overlap_ratio = get_chrloci_overlap(gf1, gf2, similarity_type=similarity_type)
            if overlap_ratio > 0:
                overlap_gf.append(overlap_ratio)
with open("2.txt", 'w') as f:
    for i in overlap_gf:
        f.write(str(i)+"\n")
hist


from lib.common.os import mkdir, rmdir, cmd_run
from api.xuyuxing.file_parser.fileIO import tsv_file_parse
import uuid
import os


def gene_mcl(input_gene_list, inflation=1.5, name_attr='name', seq_attr='model_aa_seq', work_dir='/tmp', cpu_num=5,
             force_redo=False, keep_flag=False, diamond_flag=False):
    mkdir(work_dir, True)

    if not force_redo:
        for file in os.listdir(work_dir):
            if len(file) == 32 and os.path.exists(work_dir + "/" + file + "/input.faa") and os.path.exists(
                    work_dir + "/" + file + "/all_to_all.bls") and os.path.exists(
                work_dir + "/" + file + "/subject.seq.abc") and os.path.exists(
                work_dir + "/" + file + "/subject.seq.mci") and os.path.exists(
                work_dir + "/" + file + "/subject.seq.tab"):

                tmp_dir = work_dir + "/" + file
                mcl_output = "%s/subject.%.1f.mcl.out" % (tmp_dir, inflation)

                if os.path.exists(mcl_output):
                    mcl_output_dir = tsv_file_parse(mcl_output)
                else:
                    mcl_mci_file = tmp_dir + "/subject.seq.mci"
                    mcl_tab_file = tmp_dir + "/subject.seq.tab"
                    mcl_output = "%s/subject.%.1f.mcl.out" % (tmp_dir, inflation)
                    cmd_string = "mcl %s -I %s -use-tab %s -o %s" % (mcl_mci_file, inflation, mcl_tab_file, mcl_output)
                    cmd_run(cmd_string, cwd=tmp_dir)
                    mcl_output_dir = tsv_file_parse(mcl_output)

                return mcl_output_dir

    tmp_dir = work_dir + "/" + uuid.uuid1().hex
    mkdir(tmp_dir, False)

    input_fasta_file = tmp_dir + "/input.faa"
    with open(input_fasta_file, 'w') as f:
        for gene in input_gene_list:
            f.write(">%s\n%s\n" % (getattr(gene, name_attr), getattr(gene, seq_attr)))

    # all to all blast
    if not diamond_flag:
        cmd_string = "makeblastdb -in %s -dbtype prot" % input_fasta_file
        cmd_run(cmd_string)

        all_to_all_blast = tmp_dir + "/all_to_all.bls"
        cmd_string = "blastp -query %s -db %s -out %s -evalue 1e-5 -outfmt 6 -num_threads %d" % (
            input_fasta_file, input_fasta_file, all_to_all_blast, cpu_num)
        cmd_run(cmd_string, cwd=tmp_dir)
    else:
        cmd_string = "diamond makedb --in %s --db %s" % (input_fasta_file, input_fasta_file)
        cmd_run(cmd_string)
        diamond_db = input_fasta_file + ".dmnd"

        all_to_all_blast = tmp_dir + "/all_to_all.bls"

        cmd_string = "diamond blastp -q %s -d %s -o %s -k 500 -e 1e-5 -f 6 -p %d" % (
            input_fasta_file, diamond_db, all_to_all_blast, cpu_num)
        cmd_run(cmd_string, cwd=tmp_dir)


    # mcl pre
    tsv_dict = tsv_file_parse(all_to_all_blast, fields="1,2,11")
    mcl_abc_file = tmp_dir + "/subject.seq.abc"
    with open(mcl_abc_file, "w") as f:
        for a, b, c in [tsv_dict[i] for i in tsv_dict]:
            f.write(a + "\t" + b + "\t" + c + "\n")

    # mcl
    mcl_mci_file = tmp_dir + "/subject.seq.mci"
    mcl_tab_file = tmp_dir + "/subject.seq.tab"
    cmd_string = "mcxload -abc %s --stream-mirror --stream-neg-log10 -stream-tf 'ceil(200)' -o %s -write-tab %s" % (
        mcl_abc_file, mcl_mci_file, mcl_tab_file)
    cmd_run(cmd_string, cwd=tmp_dir)

    mcl_output = "%s/subject.%.1f.mcl.out" % (tmp_dir, inflation)
    cmd_string = "mcl %s -I %s -use-tab %s -o %s" % (mcl_mci_file, inflation, mcl_tab_file, mcl_output)
    cmd_run(cmd_string, cwd=tmp_dir)
    mcl_output_dir = tsv_file_parse(mcl_output)

    if not keep_flag:
        rmdir(tmp_dir)

    return mcl_output_dir
